# The LearningOnline Network
# Documents
#
# $Id: londocs.pm,v 1.149 2004/11/11 22:47:55 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::londocs;

use strict;
use Apache::Constants qw(:common :http);
use Apache::lonnet;
use Apache::loncommon;
use Apache::lonratedt;
use Apache::lonratsrv;
use Apache::lonxml;
use Apache::loncreatecourse;
use Apache::lonnavmaps;
use HTML::Entities;
use GDBM_File;
use Apache::lonlocal;
use Cwd;

my $iconpath;

my %hash;

my $hashtied;
my %alreadyseen=();

my $hadchanges;

# Available help topics

my %help=();

# Mapread read maps into lonratedt::global arrays 
# @order and @resources, determines status
# sets @order - pointer to resources in right order
# sets @resources - array with the resources with correct idx
#

sub mapread {
    my ($coursenum,$coursedom,$map)=@_;
    return
      &Apache::lonratedt::mapread('/uploaded/'.$coursedom.'/'.$coursenum.'/'.
                                $map);
}

sub storemap {
    my ($coursenum,$coursedom,$map)=@_;
    my ($outtext,$errtext)=
      &Apache::lonratedt::storemap('/uploaded/'.$coursedom.'/'.$coursenum.'/'.
                                $map,1);
    if ($errtext) { return ($errtext,2); }
    
    $hadchanges=1;
    return ($errtext,0);
}

# ----------------------------------------- Return hash with valid author names

sub authorhosts {
    my %outhash=();
    my $home=0;
    my $other=0;
    foreach (keys %ENV) {
	if ($_=~/^user\.role\.(au|ca)\.(.+)$/) {
	    my $role=$1;
	    my $realm=$2;
	    my ($start,$end)=split(/\./,$ENV{$_});
	    if (($start) && ($start>time)) { next; }
	    if (($end) && (time>$end)) { next; }
	    my $ca; my $cd;
	    if ($1 eq 'au') {
		$ca=$ENV{'user.name'};
		$cd=$ENV{'user.domain'};
	    } else {
		($cd,$ca)=($realm=~/^\/(\w+)\/(\w+)$/);
	    }
	    my $allowed=0;
	    my $myhome=&Apache::lonnet::homeserver($ca,$cd);
	    my @ids=&Apache::lonnet::current_machine_ids();
	    foreach my $id (@ids) { if ($id eq $myhome) { $allowed=1; } }
	    if ($allowed) {
		$home++;
		$outhash{'home_'.$ca.'@'.$cd}=1;
	    } else {
		$outhash{'otherhome_'.$ca.'@'.$cd}=$myhome;
		$other++;
	    }
	}
    }
    return ($home,$other,%outhash);
}
# ------------------------------------------------------ Generate "dump" button

sub dumpbutton {
    my ($home,$other,%outhash)=&authorhosts();
    if ($home+$other==0) { return ''; }
    my $output='</td><td bgcolor="#DDDDCC">';
    if ($home) {
	return '</td><td bgcolor="#DDDDCC">'.
	    '<input type="submit" name="dumpcourse" value="'.
	    &mt('Dump Course DOCS to Construction Space').'" />'.
	    &Apache::loncommon::help_open_topic('Docs_Dump_Course_Docs');
    } else {
	return'</td><td bgcolor="#DDDDCC">'.
     &mt('Dump Course DOCS to Construction Space: available on other servers');
    }
}

# -------------------------------------------------------- Actually dump course

sub dumpcourse {
    my $r=shift;
    $r->print('<html><head><title>Dump DOCS</title></head>'.
        &Apache::loncommon::bodytag('Dump Course DOCS to Construction Space').
	      '<form name="dumpdoc" method="post">');
    my ($home,$other,%outhash)=&authorhosts();
    unless ($home) { return ''; }
    my $origcrsid=$ENV{'request.course.id'};
    my %origcrsdata=&Apache::lonnet::coursedescription($origcrsid);
    if (($ENV{'form.authorspace'}) && ($ENV{'form.authorfolder'}=~/\w/)) {
# Do the dumping
	unless ($outhash{'home_'.$ENV{'form.authorspace'}}) { return ''; }
	my ($ca,$cd)=split(/\@/,$ENV{'form.authorspace'});
	$r->print('<h3>'.&mt('Copying Files').'</h3>');
	my $title=$ENV{'form.authorfolder'};
	$title=~s/[^\w\/]+/\_/g;
	my %replacehash=();
	foreach (keys %ENV) {
	    if ($_=~/^form\.namefor\_(.+)/) {
		$replacehash{$1}=$ENV{$_};
	    }
	}
	my $crs='/uploaded/'.$ENV{'request.course.id'}.'/';
	$crs=~s/\_/\//g;
	foreach (keys %replacehash) {
	    my $newfilename=$title.'/'.$replacehash{$_};
	    $newfilename=~s/[^\w\/\.\/]+/\_/g;
	    my @dirs=split(/\//,$newfilename);
	    my $path='/home/'.$ca.'/public_html';
	    my $makepath=$path;
	    my $fail=0;
	    for (my $i=0;$i<$#dirs;$i++) {
		$makepath.='/'.$dirs[$i];
		unless (-e $makepath) { 
		    unless(mkdir($makepath,0777)) { $fail=1; } 
		}
	    }
	    $r->print('<br /><tt>'.$_.'</tt> => <tt>'.$newfilename.'</tt>: ');
	    if (my $fh=Apache::File->new('>'.$path.'/'.$newfilename)) {
		if ($_=~/\.(sequence|page|html|htm|xml|xhtml)$/) {
		    print $fh &Apache::loncreatecourse::rewritefile(
         &Apache::loncreatecourse::readfile($ENV{'request.course.id'},$_),
				     (%replacehash,$crs => '')
								    );
		} else {
		    print $fh
         &Apache::loncreatecourse::readfile($ENV{'request.course.id'},$_);
		       }
		$fh->close();
	    } else {
		$fail=1;
	    }
	    if ($fail) {
		$r->print('<font color="red">fail</font>');
	    } else {
		$r->print('<font color="green">ok</font>');
	    }
	}
    } else {
# Input form
	unless ($home==1) {
	    $r->print(
		      '<h3>'.&mt('Select the Construction Space').'</h3><select name="authorspace">');
	}
	foreach (sort keys %outhash) {
	    if ($_=~/^home_(.+)$/) {
		if ($home==1) {
		    $r->print(
		  '<input type="hidden" name="authorspace" value="'.$1.'" />');
		} else {
		    $r->print('<option value="'.$1.'">'.$1.' - '.
			      &Apache::loncommon::plainname(split(/\@/,$1)).'</option>');
		}
	    }
	}
	unless ($home==1) {
	    $r->print('</select>');
	}
	my $title=$origcrsdata{'description'};
	$title=~s/\s+/\_/gs;
	$title=~s/\W//gs;
	$r->print('<h3>'.&mt('Folder in Construction Space').'</h3><input type="text" size="50" name="authorfolder" value="'.$title.'" /><br />');
	&tiehash();
	$r->print('<h3>'.&mt('Filenames in Construction Space').'</h3><table border="2"><tr><th>'.&mt('Internal Filename').'</th><th>'.&mt('Title').'</th><th>'.&mt('Save as ...').'</th></tr>');
	foreach (&Apache::loncreatecourse::crsdirlist($origcrsid,'userfiles')) {
	    $r->print('<tr><td>'.$_.'</td>');
	    my ($ext)=($_=~/\.(\w+)$/);
	    my $title=$hash{'title_'.$hash{
		'ids_/uploaded/'.$origcrsdata{'domain'}.'/'.$origcrsdata{'num'}.'/'.$_}};
	    $r->print('<td>'.($title?$title:'&nbsp;').'</td>');
	    unless ($title) {
		$title=$_;
	    }
	    $title=~s/\.(\w+)$//;
	    $title=~s/[^\w\/]+/\_/gs;
	    $title.='.'.$ext;
	    $r->print("\n<td><input type='text' size='60' name='namefor_".$_."' value='".$title."' /></td></tr>\n");
	}
	$r->print("</table>\n");
	&untiehash();
	$r->print(
  '<p><input type="submit" name="dumpcourse" value="'.&mt('Dump Course DOCS').'" /></p></form>');
    }
}

# ------------------------------------------------------ Generate "export" button

sub exportbutton {
    return '</td><td bgcolor="#DDDDCC">'.
            '<input type="submit" name="exportcourse" value="'.
            &mt('Export Course to IMS').'" />'.
            &Apache::loncommon::help_open_topic('Docs_Export_Course_Docs');
}

sub exportcourse {
    my $r=shift;
    my %discussiontime = &Apache::lonnet::dump('discussiontimes',
                                               $ENV{'course.'.$ENV{'request.course.id'}.'.domain'}, $ENV{'course.'.$ENV{'request.course.id'}.'.num'});
    my $numdisc = keys %discussiontime;
    my $navmap = Apache::lonnavmaps::navmap->new();
    my $it=$navmap->getIterator(undef,undef,undef,1,undef,undef);
    my $curRes;
    my $outcome;

    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
                                            ['finishexport']);
    if ($ENV{'form.finishexport'}) {
        &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
                                            ['archive','discussion']);

        my @exportitems = ();
        if (defined($ENV{'form.archive'})) {
            if (ref($ENV{'form.archive'}) eq 'ARRAY') {
                @exportitems = @{$ENV{'form.archive'}};
            } else {
                $exportitems[0] = $ENV{'form.archive'};
            }
        }
        my @discussions = ();
        if (defined($ENV{'form.discussion'})) {
            if (ref($ENV{'form.discussion'}) eq 'ARRAY') {
                @discussions = $ENV{'form.discussion'};
            } else {
                $discussions[0] = $ENV{'form.discussion'};
            }
        }
        if (@exportitems == 0 && @discussions == 0) {
            $outcome = '<br />As you did not select any content items or discussions for export, an IMS package has not been created.  Please <a href="javascript:history.go(-1)">go back</a> to select either content items or discussions for export';
        } else {
            my $now = time;
            my $count = 0;
            my %symbs;
            my $manifestok = 0;
            my $imsresources;
            my $tempexport;
            my $copyresult;
            my $ims_manifest = &create_ims_store($now,\$manifestok,\$outcome,\$tempexport);
            if ($manifestok) {
                &build_package($now,$navmap,\@exportitems,\@discussions,\$outcome,\$tempexport,\$copyresult,$ims_manifest);
                close($ims_manifest);

#Create zip file in prtspool
                my $imszipfile = '/prtspool/'.
                $ENV{'user.name'}.'_'.$ENV{'user.domain'}.'_'.
                   time.'_'.rand(1000000000).'.zip';
# zip can cause an sh launch which can pass along all of %ENV
# which can be too large for /bin/sh to handle
                my %oldENV=%ENV;
                undef(%ENV);
                my $cwd = &Cwd::getcwd();
                my $imszip = '/home/httpd/'.$imszipfile;
                chdir $tempexport;
                open(OUTPUT, "zip -r $imszip *  2> /dev/null |");
                close(OUTPUT);
                chdir $cwd;
                %ENV=%oldENV;
                undef(%oldENV);
                $outcome .= 'Download the zip file from <a href="'.$imszipfile.'">IMS course archive</a><br />';
                if ($copyresult) {
                    $outcome .= 'The following errors occurred during export - '.$copyresult;
                }
            } else {
                $outcome = '<br />Unfortunately you will not be able to retrieve an IMS archive of this posts at this time, because there was a problem creating a manifest file.<br />';
            }
        }

        $r->print('<html><head><title>Export Course</title></head>'.
            &Apache::loncommon::bodytag('Export course to IMS or SCORM content package'));
        $r->print($outcome);
        $r->print('</body></html>');
    } else {
        my $display;
        $display = '<form name="exportdoc" method="post">'."\n";
        $display .= 'Choose which items you wish to export from your course.<br /><br />';
        $display .= '<table border="0" cellspacing="0" cellpadding="3">'.
                    '<tr><td><fieldset><legend>&nbsp;<b>Content items</b></legend>'.
                    '<input type="button" value="check all" '.
                    'onclick="javascript:checkAll(document.exportdoc.archive)" />'.
                    '&nbsp;&nbsp;<input type="button" value="uncheck all"'.
                    ' onclick="javascript:uncheckAll(document.exportdoc.archive)" /></fieldset></td>'.
                    '<td>&nbsp;</td><td>&nbsp;</td>'.
                    '<td align="right"><fieldset><legend>&nbsp;<b>Discussion posts'.
                    '</b></legend><input type="button" value="check all"'.
                    ' onclick="javascript:checkAll(document.exportdoc.discussion)" />'.
                    '&nbsp;&nbsp;<input type="button" value="uncheck all"'.
                    ' onclick="javascript:uncheckAll(document.exportdoc.discussion)" /></fieldset></td>'.
                    '</tr></table>';
        my $curRes;
        my $depth = 0;
        my $count = 0;
        my $boards = 0;
        my $startcount = 5;
        my %parent = ();
        my %children = ();
        my $lastcontainer = $startcount;
        my @bgcolors = ('#F6F6F6','#FFFFFF');
        $display .= '<table cellspacing="0"><tr>'.
            '<td><b>Export content item?<br /></b></td><td>&nbsp;</td><td align="right">'."\n";
        if ($numdisc > 0) {
            $display.='<b>Export&nbsp;discussion posts?</b>'."\n";
        }
        $display.='&nbsp;</td></tr>';
        while ($curRes = $it->next()) {
            if (ref($curRes)) {
                $count ++;
            }
            if ($curRes == $it->BEGIN_MAP()) {
                $depth++;
                $parent{$depth} = $lastcontainer;
            }
            if ($curRes == $it->END_MAP()) {
                $depth--;
                $lastcontainer = $parent{$depth};
            }
            if (ref($curRes)) {
                my $symb = $curRes->symb();
                my $color = $count%2;
                $display .='<tr bgcolor='.$bgcolors[$color].'><td>'."\n".
                    '<input type="checkbox" name="archive" value="'.$count.'" ';
                if (($curRes->is_sequence()) || ($curRes->is_page())) {
                    my $checkitem = $count + $boards + $startcount;
                    $display .= 'onClick="javascript:propagateCheck('."'$checkitem'".')"';
                }
                $display .= ' />'."\n";
                for (my $i=0; $i<$depth; $i++) {
                    $display .= '<img src="/adm/lonIcons/whitespace1.gif" width="25" height="1" alt="" border="0" /><img src="/adm/lonIcons/whitespace1.gif" width="25" height="1" alt="" border="0" />'."\n";
                }
                if ($curRes->is_sequence()) {
                    $display .= '<img src="/adm/lonIcons/navmap.folder.open.gif">&nbsp;'."\n";
                    $lastcontainer = $count + $startcount + $boards;
                } elsif ($curRes->is_page()) {
                    $display .= '<img src="/adm/lonIcons/navmap.page.open.gif">&nbsp;'."\n";
                    $lastcontainer = $count + $startcount + $boards;
                }
                my $currelem = $count+$boards+$startcount;
                $children{$parent{$depth}} .= $currelem.':';
                $display .= '&nbsp;'.$curRes->title().'</td>';
                if ($discussiontime{$symb} > 0) {
                    $boards ++;
                    $currelem = $count+$boards+$startcount;
                    $display .= '<td>&nbsp;</td><td align="right"><input type="checkbox" name="discussion" value="'.$count.'" />&nbsp;</td>'."\n";
                } else {
                    $display .= '<td colspan="2">&nbsp;</td>'."\n";
                }
            }
        }
        my $scripttag = qq|
<script>

function checkAll(field) {
    for (i = 0; i < field.length; i++)
        field[i].checked = true ;
}

function uncheckAll(field) {
    for (i = 0; i < field.length; i++)
        field[i].checked = false ;
}

function propagateCheck(item) {
    if (document.exportdoc.elements[item].checked == true) {
        containerCheck(item)
    }
} 

function containerCheck(item) {
    document.exportdoc.elements[item].checked = true
    var numitems = $count + $boards + $startcount
    var parents = new Array(numitems)
    for (var i=$startcount; i<numitems; i++) {
        parents[i] = new Array
    }
        |;

        foreach my $container (sort { $a <=> $b } keys %children) {
            my @contents = split/:/,$children{$container};
            for (my $i=0; $i<@contents; $i ++) {
                $scripttag .= '    parents['.$container.']['.$i.'] = '.$contents[$i]."\n";
            }
        }

        $scripttag .= qq|
    if (parents[item].length > 0) {
        for (var j=0; j<parents[item].length; j++) {
            containerCheck(parents[item][j])
        }
     }   
}

</script>
        |;
        $r->print('<html><head><title>Export Course</title>'.$scripttag.'</head>'.
            &Apache::loncommon::bodytag('Export course to IMS or SCORM content package'
));

        $r->print($display.'</table>'.
                  '<p><input type="hidden" name="finishexport" value="1">'.
                  '<input type="submit" name="exportcourse" value="'.
                  &mt('Export Course DOCS').'" /></p></form></body></html>');
    }
}

sub create_ims_store {
    my ($now,$manifestok,$outcome,$tempexport) = @_;
    $$tempexport = $Apache::lonnet::perlvar{'lonDaemons'}.'/tmp/ims_exports';
    my $ims_manifest;
    if (!-e $$tempexport) {
        mkdir($$tempexport,0700);
    }
    $$tempexport .= '/'.$now;
    if (!-e $$tempexport) {
        mkdir($$tempexport,0700);
    }
    $$tempexport .= '/'.$ENV{'user.domain'}.'_'.$ENV{'user.name'};
    if (!-e $$tempexport) {
        mkdir($$tempexport,0700);
    }
# open manifest file
    my $manifest = '/imsmanifest.xml';
    my $manifestfilename = $$tempexport.$manifest;
    if ($ims_manifest = Apache::File->new('>'.$manifestfilename)) {
        $$manifestok=1;
        print $ims_manifest
'<?xml version="1.0" encoding="UTF-8"?>'."\n".
'<manifest xmlns="http://www.imsglobal.org/xsd/imscp_v1p1"'.
' xmlns:imsmd="http://www.imsglobal.org/xsd/imsmd_v1p2"'.
' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"'.
' identifier="MANIFEST-'.$ENV{'request.course.id'}.'-'.$now.'"'.
'  xsi:schemaLocation="http://www.imsglobal.org/xsd/imscp_v1p1imscp_v1p1.xsd'.
'  http://www.imsglobal.org/xsd/imsmd_v1p2 imsmd_v1p2p2.xsd">'."\n".
'  <organizations default="ORG-."'.$ENV{'request,course.id'}.'-'.$now.'">'."\n".
'    <organization identifier="ORG-'.$ENV{'request.course.id'}.'-'.$now.'"'.
' structure="hierarchical">'."\n".
'      <title>'.$ENV{'request.'.$ENV{'request.course.id'}.'.description'}.'</title>'
    } else {
        $$outcome .= 'An error occurred opening the IMS manifest file.<br />'
;
    }
    return $ims_manifest;
}

sub build_package {
    my ($now,$navmap,$exportitems,$discussions,$outcome,$tempexport,$copyresult,$ims_manifest) = @_;
# first iterator to look for dependencies
    my $it = $navmap->getIterator(undef,undef,undef,1,undef,undef);
    my $curRes;
    my $count = 0;
    my $depth = 0;
    my $lastcontainer = 0;
    my %parent = ();
    my @dependencies = ();
    my $cnum = $ENV{'request.'.$ENV{'request.course.id'}.'.num'};
    my $cdom = $ENV{'request.'.$ENV{'request.course.id'}.'.domain'};
    while ($curRes = $it->next()) {
        if (ref($curRes)) {
            $count ++;
        }
        if ($curRes == $it->BEGIN_MAP()) {
            $depth++;
            $parent{$depth} = $lastcontainer;
        }
        if ($curRes == $it->END_MAP()) {
            $depth--;
            $lastcontainer = $parent{$depth};
        }
        if (ref($curRes)) {
            if ($curRes->is_sequence() || $curRes->is_page()) {
                $lastcontainer = $count;
            }
            if (grep/^$count$/,@$exportitems) {
                &get_dependencies($exportitems,\%parent,$depth,\@dependencies);
            }
        }
    }
# second iterator to build manifest and store resources
    $it = $navmap->getIterator(undef,undef,undef,1,undef,undef);
    $depth = 0;
    my $prevdepth;
    $count = 0;
    my $imsresources;
    my $pkgdepth;
    if ($curRes == $it->BEGIN_MAP()) {
        $prevdepth = $depth;
        $depth++;
    }
    if ($curRes == $it->END_MAP()) {
        $prevdepth = $depth;
        $depth--;
    }

    if (ref($curRes)) {
        if ((grep/^$count$/,@$exportitems) || (grep/^$count$/,@dependencies)) {
            my $symb = $curRes->symb();
            my $isvisible = 'true';
            my $resourceref;
            if ($curRes->randomout()) {
                $isvisible = 'false';
            }
            unless ($curRes->is_sequence()) {
                $resourceref = 'identifierref="RES-'.$ENV{'request.course.id'}.'-'.$count.'"';
            }
            if (($depth <= $prevdepth) && ($count > 1)) {
                print $ims_manifest '  </item>'."\n";
            }
            $prevdepth = $depth;

            my $itementry =
              '<item identifier="ITEM-'.$ENV{'request.course.id'}.'-'.$count.
              '" isvisible="'.$isvisible.'" '.$resourceref.'>'.
              '<title>'.$curRes->title().'</title>';
            print $ims_manifest "\n".$itementry;

            unless ($curRes->is_sequence()) {
                my $content_file;
                my @hrefs = ();
                &process_content($count,$curRes,$cdom,$cnum,$symb,$content_file,\@hrefs,$copyresult,$tempexport);
                if ($content_file) {
                    $imsresources .= "\n".
                     '   <resource identifier="RES-'.$ENV{'request.course.id'}.'-'.$count.
                     '" type="webcontent" href="'.$content_file.'">'."\n".
                     '       <file href="'.$content_file.'" />'."\n";
                    foreach (@hrefs) {
                        $imsresources .=
                     '        <file href="'.$_.'" />'."\n";
                    }
                    $imsresources .= '    </resource>'."\n";
                }
            }
            $pkgdepth = $depth;
        }
    }
    while ($pkgdepth > -1) {
        print $ims_manifest "    </item>\n";
        $pkgdepth --;
    }
    my $resource_text = qq|
    </organization>
  </organizations>
  <resources>
    $imsresources
  </resources>
</manifest>
    |;
    print $ims_manifest $resource_text;
}

sub get_dependencies {
    my ($exportitems,$parent,$depth,$dependencies) = @_;
    if ($depth > 1) {
        unless (grep/^$$parent{$depth}$/,@$exportitems || grep/^$$parent{$depth}$/,@$dependencies) {
            push @$dependencies, $$parent{$depth};
            if ($depth > 2) {
                &get_dependencies($exportitems,$parent,$depth-1,$dependencies);
            }
        }
    }
}

sub process_content {
    my ($count,$curRes,$cdom,$cnum,$symb,$content_file,$href,$copyresult,$tempexport) = @_;
    my $content_type;
    my $message;
# find where user is author or co-author
    my %roleshash = &Appache::lonnet::get_my_roles();
    if ($curRes->is_page()) {
        $content_type = 'page';
    } elsif ($symb =~ m-public/$cdom/$cnum/syllabus$-) {
        $content_type = 'syllabus';
    } elsif ($symb =~ m-\.sequence____\d+____ext-) {
        $content_type = 'external';
    } elsif ($symb =~ m-adm/navmaps$-) {
        $content_type =  'navmap';
    } elsif ($symb =~ m-adm/$cdom/$cnum/\d+/smppg$-) {
        $content_type = 'simplepage';
    } elsif ($symb =~ m-$-) {
        $content_type = 'simpleproblem';
    } elsif ($symb =~ m-adm/$cdom/$cnum/\d+/bulletinboard$-) {
        $content_type = 'bulletinboard';
    } elsif ($symb =~ m-adm/$cdom/$cnum/\d+/aboutme$-) {
        $content_type = 'aboutme';
    } elsif ($symb =~ m-uploaded/$cdom/$cnum-) {
        &replicate_content($cdom,$cnum,$tempexport,$symb,$count,\$message,'uploaded');
    } elsif ($symb =~ m-\.sequence____\d+____([^/])/([^/])-) {
        my $coauth = $2.':'.$1.':ca';
        my $canedit = 0;
        if ($1 eq $ENV{'user.domain'} && $2 eq $ENV{'user.name'})  {
            $canedit= 1;
        } elsif (defined($roleshash{$coauth})) {
            if ($roleshash{$coauth} =~ /(\d+):(\d+)/) {
                if (($1 < time || $1 == 0) && ($2 == 0 || $2 >= time)) {
                    $canedit = 1;
                }
            } elsif ($roleshash{$coauth} eq ':') {
                $canedit = 1;
            }
        }
        if ($canedit) {
            &replicate_content($cdom,$cnum,$tempexport,$symb,$count,\$message,'resource');
        } else {
            &replicate_content($cdom,$cnum,$tempexport,$symb,$count,\$message,'noedit');
        }
    }
    $$copyresult .= $message."\n";
}

sub replicate_content {
    my ($cdom,$cnum,$tempexport,$symb,$count,$message,$caller) = @_;
    my ($map,$ind,$url)=&Apache::lonnet::decode_symb($symb);
    my $feedurl = &Apache::lonnet::clutter($url);

    my $content;
    my $filename;
    my $repstatus;
    if ($url =~ m-[^/]/(.+)$-) {
        $filename = $1;
        if (!-e $tempexport.'/resources') {
            mkdir($tempexport.'/resources',0700);
        }
        if (!-e $tempexport.'/resources') {
            mkdir($tempexport.'/resources/'.$count,0700);
        }
        my $destination = $$tempexport.'/resources/'.$count.'/'.$filename;
        my $copiedfile;
        if ($copiedfile = Apache::File->new('>'.$destination)) {
            my $content;
            if ($caller eq 'uploaded' || $caller eq 'resource') {
                $content = &Apache::lonnet::getfile($url);
                if ($content eq -1) {
                    $$message = 'Could not copy file '.$filename;
                } else {
                    $repstatus = 'ok';
                }
            } elsif ($caller eq 'noedit') {
                my $rtncode;
                $repstatus = &getuploaded('GET',$url,$cdom,$cnum,$content,$rtncode);
                unless ($repstatus eq 'ok') {
                    $$message = 'Could not render '.$url.' server message - '.$rtncode;
                }
            }
            if ($repstatus eq 'ok') {
                print $copiedfile $content;
            }
            close($copiedfile);
        } else {
            $$message = 'Could not open destination file for '.$filename."\n";
        }
    } else {
        $$message = 'Could not determine name of file for ';
    }
    return $repstatus;
}

# Imports the given (name, url) resources into the course
# coursenum, coursedom, and folder must precede the list
sub group_import {
    my $coursenum = shift;
    my $coursedom = shift;
    my $folder = shift;
    my $container = shift;
    my $caller = shift;
    while (@_) {
	my $name = shift;
	my $url = shift;
        if (($url =~ m#^/uploaded/$coursedom/$coursenum/(default_\d+\.)(page|sequence)$#) && ($caller eq 'londocs')) {
            my $errtext = '';
            my $fatal = 0;
            my $newmapstr = '<map>'."\n".
                            '<resource id="1" src="" type="start"></resource>'."\n".
                            '<link from="1" to="2" index="1"></link>'."\n".
                            '<resource id="2" src="" type="finish"></resource>'."\n".
                            '</map>';
            $ENV{'form.output'}=$newmapstr;
            my $home=&Apache::lonnet::homeserver($coursenum,$coursedom);
            my $result=&Apache::lonnet::finishuserfileupload($coursenum,$coursedom,$home,
                                                'output',$1.$2);
            if ($result != m|^/uploaded/|) {
                $errtext.='Map not saved: A network error occured when trying to save the new map. ';
                $fatal = 2;
            }
            if ($fatal) {
                return ($errtext,$fatal);
            }
        }
	if ($url) {
	    my $idx = $#Apache::lonratedt::resources + 1;
	    $Apache::lonratedt::order[$#Apache::lonratedt::order+1]=$idx;
	    my $ext = 'false';
	    if ($url=~/^http:\/\//) { $ext = 'true'; }
	    $url =~ s/:/\&colon;/g;
	    $name =~ s/:/\&colon;/g;
	    $Apache::lonratedt::resources[$idx] = 
		join ':', ($name, $url, $ext, 'normal', 'res');
	}
    }
    return &storemap($coursenum, $coursedom, $folder.'.'.$container);
}

sub breadcrumbs {
    my ($where)=@_;
    &Apache::lonhtmlcommon::clear_breadcrumbs();
    my (@folders);
    if ($ENV{'form.pagepath'}) {
        @folders = split('&',$ENV{'form.pagepath'});
    } else {
        @folders=split('&',$ENV{'form.folderpath'});
    }
    my $folderpath;
    while (@folders) {
	my $folder=shift(@folders);
	my $foldername=shift(@folders);
	if ($folderpath) {$folderpath.='&';}
	$folderpath.=$folder.'&'.$foldername;
	my $url='/adm/coursedocs?folderpath='.
	    &Apache::lonnet::escape($folderpath);
	    &Apache::lonhtmlcommon::add_breadcrumb(
		      {'href'=>$url,
		       'title'=>&Apache::lonnet::unescape($foldername),
		       'text'=>'<font size="+1">'.
			   &Apache::lonnet::unescape($foldername).'</font>'
		       });
		       
						 
    }
    return &Apache::lonhtmlcommon::breadcrumbs(undef,undef,undef,undef,undef,
					       0,'nohelp');
}

sub editor {
    my ($r,$coursenum,$coursedom,$folder,$allowed)=@_;

    $r->print(&breadcrumbs($folder));
    my $errtext='';
    my $fatal=0;
    my $container='sequence';
    if ($ENV{'form.pagepath'}) {
        $container='page';
    }
    ($errtext,$fatal)=
              &mapread($coursenum,$coursedom,$folder.'.'.$container);
    if ($#Apache::lonratedt::order<1) {
       	$Apache::lonratedt::order[0]=1;
        $Apache::lonratedt::resources[1]='';
    }
    if ($fatal) {
	   $r->print('<p><font color="red">'.$errtext.'</font></p>');
    } else {
# ------------------------------------------------------------ Process commands

# ---------------- if they are for this folder and user allowed to make changes
	if (($allowed) && ($ENV{'form.folder'} eq $folder)) {
# set parameters and change order
	    if (defined($ENV{'form.setparms'})) {
		my $idx=$ENV{'form.setparms'};
# set parameters
		if ($ENV{'form.randpick_'.$idx}) {
		    &Apache::lonratedt::storeparameter($idx,'parameter_randompick',$ENV{'form.randpick_'.$idx},'int_pos');
		} else {
		    &Apache::lonratedt::delparameter($idx,'parameter_randompick');
		}
		if ($ENV{'form.hidprs_'.$idx}) {
		    &Apache::lonratedt::storeparameter($idx,'parameter_hiddenresource','yes','string_yesno');
		} else {
		    &Apache::lonratedt::delparameter($idx,'parameter_hiddenresource');
		}
		if ($ENV{'form.encprs_'.$idx}) {
		    &Apache::lonratedt::storeparameter($idx,'parameter_encrypturl','yes','string_yesno');
		} else {
		    &Apache::lonratedt::delparameter($idx,'parameter_encrypturl');
		}

		if ($ENV{'form.newpos'}) {
# change order

		    my $newpos=$ENV{'form.newpos'}-1;
		    my $currentpos=$ENV{'form.currentpos'}-1;
		    my $i;
		    my @neworder=();
		    if ($newpos>$currentpos) {
# moving stuff up
			for ($i=0;$i<$currentpos;$i++) {
			    $neworder[$i]=$Apache::lonratedt::order[$i];
			}
			for ($i=$currentpos;$i<$newpos;$i++) {
			    $neworder[$i]=$Apache::lonratedt::order[$i+1];
			}
                        $neworder[$newpos]=$Apache::lonratedt::order[$currentpos];
			for ($i=$newpos+1;$i<=$#Apache::lonratedt::order;$i++) {
			    $neworder[$i]=$Apache::lonratedt::order[$i];
			}
		    } else {
# moving stuff down
			for ($i=0;$i<$newpos;$i++) {
			    $neworder[$i]=$Apache::lonratedt::order[$i];
			}
			$neworder[$newpos]=$Apache::lonratedt::order[$currentpos];
			for ($i=$newpos+1;$i<$currentpos+1;$i++) {
			    $neworder[$i]=$Apache::lonratedt::order[$i-1];
			}
			for ($i=$currentpos+1;$i<=$#Apache::lonratedt::order;$i++) {
			    $neworder[$i]=$Apache::lonratedt::order[$i];
			}
		    }
		    @Apache::lonratedt::order=@neworder;
		}
# store the changed version

		($errtext,$fatal)=&storemap($coursenum,$coursedom,$folder.'.'.$container);
		if ($fatal) {
		    $r->print('<p><font color="red">'.$errtext.'</font></p>');
		    return;
		}
		
	    }

# upload a file, if present
           if (($ENV{'form.uploaddoc.filename'}) &&
               ($ENV{'form.cmd'}=~/^upload_(\w+)/)) {
	    if ( ($folder=~/^$1/) || ($1 eq 'default') ) {
# this is for a course, not a user, so set coursedoc flag
# probably the only place in the system where this should be "1"
	      my $url=&Apache::lonnet::userfileupload('uploaddoc',1,'docs');
              my $ext='false';
              if ($url=~/^http\:\/\//) { $ext='true'; }
              $url=~s/\:/\&colon;/g;
	      my $comment=$ENV{'form.comment'};
              $comment=~s/\</\&lt\;/g;
              $comment=~s/\>/\&gt\;/g;
              $comment=~s/\:/\&colon;/g;
              if ($folder=~/^supplemental/) {
		  $comment=time.'___&&&___'.$ENV{'user.name'}.'___&&&___'.
		      $ENV{'user.domain'}.'___&&&___'.$comment;
              }
              my $newidx=$#Apache::lonratedt::resources+1;
              $Apache::lonratedt::resources[$newidx]=
                  $comment.':'.$url.':'.$ext.':normal:res';
              $Apache::lonratedt::order[$#Apache::lonratedt::order+1]=
                                                              $newidx;       

	      ($errtext,$fatal)=&storemap($coursenum,$coursedom,$folder.'.'.$container);
	      if ($fatal) {
		  $r->print('<p><font color="red">'.$errtext.'</font></p>');
		  return;
	      }
	     }
            }
	    if ($ENV{'form.cmd'}) {
                my ($cmd,$idx)=split(/\_/,$ENV{'form.cmd'});
                if ($cmd eq 'del') {
		    my (undef,$url)=split(':',$Apache::lonratedt::resources[$Apache::lonratedt::order[$idx]]);
		    if ($url=~m|/+uploaded/\Q$coursedom\E/\Q$coursenum\E/|) {
			&Apache::lonnet::removeuploadedurl($url);
		    }
		    for (my $i=$idx;$i<$#Apache::lonratedt::order;$i++) {
                        $Apache::lonratedt::order[$i]=
                          $Apache::lonratedt::order[$i+1];
                    }
                    $#Apache::lonratedt::order--;
                } elsif ($cmd eq 'up') {
		  if (($idx) && (defined($Apache::lonratedt::order[$idx-1]))) {
                    my $i=$Apache::lonratedt::order[$idx-1];
                    $Apache::lonratedt::order[$idx-1]=
			$Apache::lonratedt::order[$idx];
                    $Apache::lonratedt::order[$idx]=$i;
		   }
                } elsif ($cmd eq 'down') {
		   if (defined($Apache::lonratedt::order[$idx+1])) {
                    my $i=$Apache::lonratedt::order[$idx+1];
                    $Apache::lonratedt::order[$idx+1]=
			$Apache::lonratedt::order[$idx];
                    $Apache::lonratedt::order[$idx]=$i;
		   }
                } elsif ($cmd eq 'rename') {
                    my $ratstr = $Apache::lonratedt::resources[$Apache::lonratedt::order[$idx]];
                    my ($rtitle,@rrest)=split(/\:/,
                       $Apache::lonratedt::resources[
				       $Apache::lonratedt::order[$idx]]);
                    my $comment=
                     &HTML::Entities::decode($ENV{'form.title'});
                    $comment=~s/\</\&lt\;/g;
                    $comment=~s/\>/\&gt\;/g;
                    $comment=~s/\:/\&colon;/g;
		    if ($comment=~/\S/) {
			$Apache::lonratedt::resources[
				       $Apache::lonratedt::order[$idx]]=
				            $comment.':'.join(':',@rrest);
		    }
                }
# Store the changed version
		($errtext,$fatal)=&storemap($coursenum,$coursedom,
					    $folder.'.'.$container);
		if ($fatal) {
		    $r->print('<p><font color="red">'.$errtext.'</font></p>');
		    return;
		}
            }
# Group import/search
	    if ($ENV{'form.importdetail'}) {
		my @imports;
		foreach (split(/\&/,$ENV{'form.importdetail'})) {
		    if (defined($_)) {
			my ($name,$url)=split(/\=/,$_);
			$name=&Apache::lonnet::unescape($name);
			$url=&Apache::lonnet::unescape($url);
			push @imports, $name, $url;
		    }
		}
# Store the changed version
		($errtext,$fatal)=group_import($coursenum, $coursedom, $folder,
					       $container,'londocs',@imports);
		if ($fatal) {
		    $r->print('<p><font color="red">'.$errtext.'</font></p>');
		    return;
		}
            }
# Loading a complete map
	   if (($ENV{'form.importmap'}) && ($ENV{'form.loadmap'})) {
	       foreach (&Apache::lonsequence::attemptread(&Apache::lonnet::filelocation('',$ENV{'form.importmap'}))) {
                   my $idx=$#Apache::lonratedt::resources;
                   $idx++;
                   $Apache::lonratedt::resources[$idx]=$_;
                   $Apache::lonratedt::order
		       [$#Apache::lonratedt::order+1]=$idx;
	       }

# Store the changed version
	       ($errtext,$fatal)=&storemap($coursenum,$coursedom,
					   $folder.'.'.$container);
	       if ($fatal) {
		   $r->print('<p><font color="red">'.$errtext.'</font></p>');
		   return;
	       }
           }
       }
# ---------------------------------------------------------------- End commands
# ---------------------------------------------------------------- Print screen
        my $idx=0;
	my $shown=0;
        $r->print('<table>');
        foreach (@Apache::lonratedt::order) {
           my ($name,$url)=split(/\:/,$Apache::lonratedt::resources[$_]);
           unless ($name) {  $name=(split(/\//,$url))[-1]; }
           unless ($name) { $idx++; next; }
           $r->print(&entryline($idx,$name,$url,$folder,$allowed,$_,$coursenum));
           $idx++;
	   $shown++;
        }
	unless ($shown) {
	    $r->print('<tr><td>'.&mt('Currently no documents.').'</td></tr>');
	}
        $r->print('</table>');
    }
}

# --------------------------------------------------------------- An entry line

sub entryline {
    my ($index,$title,$url,$folder,$allowed,$residx,$coursenum)=@_;
    $title=~s/\&colon\;/\:/g;
    $title=&HTML::Entities::encode(&HTML::Entities::decode(
     &Apache::lonnet::unescape($title)),'"<>&\'');
    my $renametitle=$title;
    my $foldertitle=$title;
    my $pagetitle=$title;
    my $orderidx=$Apache::lonratedt::order[$index];
    if ($title=~ /^(\d+)___&amp;&amp;&amp;___(\w+)___&amp;&amp;&amp;___(\w+)___&amp;&amp;&amp;___(.*)$/	) { 
	$foldertitle=&Apache::lontexconvert::msgtexconverted($4);
	$renametitle=$4;
	$title='<i>'.&Apache::lonlocal::locallocaltime($1).'</i> '.
	    &Apache::loncommon::plainname($2,$3).': <br />'.
	    $foldertitle;
    }
    $renametitle=~s/\&quot\;/\\\"/g;
    my $line='<tr>';
# Edit commands
    my $container;
    my $folderpath;
    if ($ENV{'form.folderpath'}) {
        $container = 'sequence';
	$folderpath=&Apache::lonnet::escape($ENV{'form.folderpath'});
	# $htmlfoldername=&HTML::Entities::encode($ENV{'form.foldername'},'<>&"');
    }
    my $pagepath;
    if ($ENV{'form.pagepath'}) {
        $container = 'page';
        $pagepath=&Apache::lonnet::escape($ENV{'form.pagepath'});
    }
    if ($allowed) {
	my $incindex=$index+1;
	my $selectbox='';
	if ($folder!~/^supplemental/) {
	    $selectbox=
		'<input type="hidden" name="currentpos" value="'.$incindex.'" />'.
		'<select name="newpos" onChange="this.form.submit()">';
	    for (my $i=1;$i<=$#Apache::lonratedt::order+1;$i++) {
		if ($i==$incindex) {
		    $selectbox.='<option value="" selected="1">('.$i.')</option>';
		} else {
		    $selectbox.='<option value="'.$i.'">'.$i.'</option>';
		}
	    }
	    $selectbox.='</select>';
	}
	my %lt=&Apache::lonlocal::texthash(
                'up' => 'Move Up',
		'dw' => 'Move Down',
		'rm' => 'Remove',
		'rn' => 'Rename');
        if ($ENV{'form.pagepath'}) {
            $line.=(<<END);
<form name="entry_$index" action="/adm/coursedocs" method="post">
<input type="hidden" name="pagepath" value="$ENV{'form.pagepath'}" />
<input type="hidden" name="setparms" value="$orderidx" />
<td><table border='0' cellspacing='2' cellpadding='0'>
<tr><td bgcolor="#DDDDDD">
<a href='/adm/coursedocs?cmd=up_$index&pagepath=$pagepath'>
<img src="${iconpath}move_up.gif" alt='$lt{'up'}' border='0' /></a></td></tr>
<tr><td bgcolor="#DDDDDD">
<a href='/adm/coursedocs?cmd=down_$index&pagepath=$pagepath'>
<img src="${iconpath}move_down.gif" alt='$lt{'dw'}' border='0' /></a></td></tr>
</table></td>
<td>$selectbox
</td><td bgcolor="#DDDDDD">
<a href='javascript:removeres("$pagepath","$index","$renametitle","page");'>
<font size="-2" color="#990000">$lt{'rm'}</font></a>
<a href='javascript:changename("$pagepath","$index","$renametitle","page");'>
<font size="-2" color="#009900">$lt{'rn'}</font></a></td>
END
        } else {
            $line.=(<<END); 
<form name="entry_$index" action="/adm/coursedocs" method="post">
<input type="hidden" name="folderpath" value="$ENV{'form.folderpath'}" />
<input type="hidden" name="setparms" value="$orderidx" />
<td><table border='0' cellspacing='2' cellpadding='0'>
<tr><td bgcolor="#DDDDDD">
<a href='/adm/coursedocs?cmd=up_$index&folderpath=$folderpath'>
<img src="${iconpath}move_up.gif" alt='$lt{'up'}' border='0' /></a></td></tr>
<tr><td bgcolor="#DDDDDD">
<a href='/adm/coursedocs?cmd=down_$index&folderpath=$folderpath'>
<img src="${iconpath}move_down.gif" alt='$lt{'dw'}' border='0' /></a></td></tr>
</table></td>
<td>$selectbox
</td><td bgcolor="#DDDDDD">
<a href='javascript:removeres("$folderpath","$index","$renametitle","sequence");'>
<font size="-2" color="#990000">$lt{'rm'}</font></a>
<a href='javascript:changename("$folderpath","$index","$renametitle","sequence");'>
<font size="-2" color="#009900">$lt{'rn'}</font></a></td>
END
        }
    }
# Figure out what kind of a resource this is
    my ($extension)=($url=~/\.(\w+)$/);
    my $uploaded=($url=~/^\/*uploaded\//);
    my $icon=&Apache::loncommon::icon($url);
    my $isfolder=0;
    my $ispage=0;
    my $folderarg;
    my $pagearg;
    my $pagefile;
    if ($uploaded) {
	if ($extension eq 'sequence') {
	    $icon=$iconpath.'/folder_closed.gif';
	    $url=~/$coursenum\/([\/\w]+)\.sequence$/;
	    $url='/adm/coursedocs?';
	    $folderarg=$1;
	    $isfolder=1;
        } elsif ($extension eq 'page') {
            $icon=$iconpath.'/page.gif';
            $url=~/$coursenum\/([\/\w]+)\.page$/;
            $pagearg=$1;
            $url='/adm/coursedocs?';
            $ispage=1;
	} else {
	    &Apache::lonnet::allowuploaded('/adm/coursedoc',$url);
	}
    }
    $url=~s/^http\&colon\;\/\//\/adm\/wrapper\/ext\//;
    if ((!$isfolder) && ($residx) && ($folder!~/supplemental/) && (!$ispage)) {
	my $symb=&Apache::lonnet::symbclean(
          &Apache::lonnet::declutter('uploaded/'.
           $ENV{'course.'.$ENV{'request.course.id'}.'.domain'}.'/'.
           $ENV{'course.'.$ENV{'request.course.id'}.'.num'}.'/'.$folder.
           '.sequence').
           '___'.$residx.'___'.
	   &Apache::lonnet::declutter($url));
	(undef,undef,$url)=&Apache::lonnet::decode_symb($symb);
	$url=&Apache::lonnet::clutter($url);
	if ($url=~/^\/*uploaded\//) {
	    $url=~/\.(\w+)$/;
	    my $embstyle=&Apache::loncommon::fileembstyle($1);
	    if (($embstyle eq 'img') || ($embstyle eq 'emb')) {
		$url='/adm/wrapper'.$url;
	    } elsif ($embstyle eq 'ssi') {
		#do nothing with these
	    } elsif ($url!~/\.(sequence|page)$/) {
		$url='/adm/coursedocs/showdoc'.$url;
	    }
	} elsif ($url=~m|^/ext/|) { 
	    $url='/adm/wrapper'.$url;
	}
        unless ($container eq 'page') {
	    $url.=(($url=~/\?/)?'&':'?').'symb='.&Apache::lonnet::escape($symb);
        }
    }
    my $parameterset='&nbsp;';
    if ($isfolder) {
	my $foldername=&Apache::lonnet::escape($foldertitle);
	my $folderpath=$ENV{'form.folderpath'};
	if ($folderpath) { $folderpath.='&' };
	$folderpath.=$folderarg.'&'.$foldername;
	$url.='folderpath='.&Apache::lonnet::escape($folderpath);
	$parameterset='<label>'.&mt('Randomly Pick: ').
	    '<input type="text" size="4" name="randpick_'.$orderidx.'" value="'.
	    (&Apache::lonratedt::getparameter($orderidx,
                                              'parameter_randompick'))[0].
                                              '" />'.'</label>';
       
    }
    if ($ispage) {
        my $pagename=&Apache::lonnet::escape($pagetitle);
        my $pagepath;
        my $folderpath=$ENV{'form.folderpath'};
        if ($folderpath) { $pagepath = $folderpath.'&' };
        $pagepath.=$pagearg.'&'.$pagename;
        $url.='pagepath='.&Apache::lonnet::escape($pagepath);
    }
    $line.='<td bgcolor="#FFFFBB"><a href="'.$url.'"><img src="'.$icon.
	'" border="0"></a></td>'.
        "<td bgcolor='#FFFFBB'><a href='$url'>$title</a></td>";
    if (($allowed) && ($folder!~/^supplemental/)) {
 	my %lt=&Apache::lonlocal::texthash(
 			      'hd' => 'Hidden',
 			      'ec' => 'URL hidden',
 			      'sp' => 'Store Parameters');
	my $enctext=
	    ((&Apache::lonratedt::getparameter($orderidx,'parameter_encrypturl'))[0]=~/^yes$/i?' checked="1"':'');
	my $hidtext=
	    ((&Apache::lonratedt::getparameter($orderidx,'parameter_hiddenresource'))[0]=~/^yes$/i?' checked="1"':'');
	$line.=(<<ENDPARMS);
<td bgcolor="#BBBBFF"><font size='-2'>
<nobr><label><input type="checkbox" name="hidprs_$orderidx" $hidtext/> $lt{'hd'}</label></nobr></td>
<td bgcolor="#BBBBFF"><font size='-2'>
<nobr><label><input type="checkbox" name="encprs_$orderidx" $enctext/> $lt{'ec'}</label></nobr></td>
<td bgcolor="#BBBBFF"><font size="-2">$parameterset</font></td>
<td bgcolor="#BBBBFF"><font size='-2'>
<input type="submit" value="$lt{'sp'}" />
</font></td>
ENDPARMS
    }
    $line.="</form></tr>";
    return $line;
}

# ---------------------------------------------------------------- tie the hash

sub tiehash {
    my ($mode)=@_;
    $hashtied=0;
    &Apache::lonnet::logthis(" tiehash1 -- $hashtied ");
    if ($ENV{'request.course.fn'}) {
	&Apache::lonnet::logthis(" tiehash2 -- $hashtied ");
	if ($mode eq 'write') {
	    &Apache::lonnet::logthis(" tiehash3 -- $hashtied ");
	    if (tie(%hash,'GDBM_File',$ENV{'request.course.fn'}.".db",
		    &GDBM_WRCREAT(),0640)) {
                $hashtied=2;
		&Apache::lonnet::logthis(" tiehash4 -- $hashtied ");
	    }
	} else {
	    &Apache::lonnet::logthis(" tiehash5 -- $hashtied ");
	    if (tie(%hash,'GDBM_File',$ENV{'request.course.fn'}.".db",
		    &GDBM_READER(),0640)) {
                $hashtied=1;
		&Apache::lonnet::logthis(" tiehash6 -- $hashtied ");
	    }
	}
    }    
    &Apache::lonnet::logthis(" tiehash7 -- $hashtied ");
}

sub untiehash {
    &Apache::lonnet::logthis(" untiehash1 -- $hashtied ");
    if ($hashtied) { untie %hash; }
    $hashtied=0;
    &Apache::lonnet::logthis(" untiehash2 -- $hashtied ");
}

# --------------------------------------------------------------- check on this

sub checkonthis {
    my ($r,$url,$level,$title)=@_;
    $url=&Apache::lonnet::unescape($url);
    $alreadyseen{$url}=1;
    $r->rflush();
    if (($url) && ($url!~/^\/uploaded\//) && ($url!~/\*$/)) {
       $r->print("\n<br />");
       for (my $i=0;$i<=$level*5;$i++) {
           $r->print('&nbsp;');
       }
       $r->print('<a href="'.$url.'" target="cat">'.
		 ($title?$title:$url).'</a> ');
       if ($url=~/^\/res\//) {
	  my $result=&Apache::lonnet::repcopy(
                              &Apache::lonnet::filelocation('',$url));
          if ($result==OK) {
             $r->print('<font color="green">'.&mt('ok').'</font>');
             $r->rflush();
             &Apache::lonnet::countacc($url);
             $url=~/\.(\w+)$/;
             if (&Apache::loncommon::fileembstyle($1) eq 'ssi') {
		 $r->print('<br />');
                 $r->rflush();
                 for (my $i=0;$i<=$level*5;$i++) {
                     $r->print('&nbsp;');
                 }
                 $r->print('- '.&mt('Rendering').': ');
                 my $oldpath=$ENV{'request.filename'};
                 $ENV{'request.filename'}=&Apache::lonnet::filelocation('',$url);
                 &Apache::lonxml::xmlparse($r,'web',
                   &Apache::lonnet::getfile(
                    &Apache::lonnet::filelocation('',$url)));
		 undef($Apache::lonhomework::parsing_a_problem);
		 $ENV{'request.filename'}=$oldpath;
                 if (($Apache::lonxml::errorcount) ||
                     ($Apache::lonxml::warningcount)) {
		     if ($Apache::lonxml::errorcount) {
                        $r->print('<img src="/adm/lonMisc/bomb.gif" /><font color="red"><b>'.
			  $Apache::lonxml::errorcount.' '.
				  &mt('error(s)').'</b></font> ');
                     }
		     if ($Apache::lonxml::warningcount) {
                        $r->print('<font color="blue">'.
			  $Apache::lonxml::warningcount.' '.
				  &mt('warning(s)').'</font>');
                     }
                 } else {
                     $r->print('<font color="green">'.&mt('ok').'</font>');
                 }
                 $r->rflush();
             }
	     my $dependencies=
                &Apache::lonnet::metadata($url,'dependencies');
             foreach (split(/\,/,$dependencies)) {
		 if (($_=~/^\/res\//) && (!$alreadyseen{$_})) {
                    &checkonthis($r,$_,$level+1);
                 }
             }
          } elsif ($result==HTTP_SERVICE_UNAVAILABLE) {
             $r->print('<font color="red"><b>'.&mt('connection down').'</b></font>');
          } elsif ($result==HTTP_NOT_FOUND) {
	      unless ($url=~/\$/) {
		  $r->print('<font color="red"><b>'.&mt('not found').'</b></font>');
	      } else {
		  $r->print('<font color="yellow"><b>'.&mt('unable to verify variable URL').'</b></font>');
	      }
          } else {
             $r->print('<font color="red"><b>'.&mt('access denied').'</b></font>');
          }
      }
   }
}


#
# -------------------------------------------------------------- Verify Content
# 
sub verifycontent {
   my $r=shift; 
   my $loaderror=&Apache::lonnet::overloaderror($r);
   if ($loaderror) { return $loaderror; }

   $r->print('<html><head><title>Verify Content</title></head>'.
              &Apache::loncommon::bodytag('Verify Course Documents'));
   $hashtied=0;
   undef %alreadyseen;
   %alreadyseen=();
   &tiehash();
   foreach (keys %hash) {
       if ($hash{$_}=~/\.(page|sequence)$/) {
	   if (($_=~/^src_/) && ($alreadyseen{&Apache::lonnet::unescape($hash{$_})})) {
	       $r->print('<hr /><font color="red">'.
			 &mt('The following sequence or page is included more than once in your course: ').
			 &Apache::lonnet::unescape($hash{$_}).'</font><br />'.
			 &mt('Note that grading records for problems included in this sequence or folder will overlap.<hr />'));
	   }
       }
       if (($_=~/^src\_(.+)$/) && (!$alreadyseen{&Apache::lonnet::unescape($hash{$_})})) {
           &checkonthis($r,$hash{$_},0,$hash{'title_'.$1});
       }
   }
   &untiehash();
   $r->print('<h1>'.&mt('Done').'.</h1>'.'<a href="/adm/coursedocs">'.
	     &mt('Return to DOCS').'</a>');
}

# -------------------------------------------------------------- Check Versions

sub checkversions {
    my $r=shift;
    $r->print('<html><head><title>Check Versions</title></head>'.
              &Apache::loncommon::bodytag('Check Course Document Versions'));
    my $header='';
    my $startsel='';
    my $monthsel='';
    my $weeksel='';
    my $daysel='';
    my $allsel='';
    my %changes=();
    my $starttime=0;
    my $haschanged=0;
    my %setversions=&Apache::lonnet::dump('resourceversions',
			  $ENV{'course.'.$ENV{'request.course.id'}.'.domain'},
			  $ENV{'course.'.$ENV{'request.course.id'}.'.num'});

    $hashtied=0;
    &tiehash();
    my %newsetversions=();
    if ($ENV{'form.setmostrecent'}) {
	$haschanged=1;
	foreach (keys %hash) {
	    if ($_=~/^ids\_(\/res\/.+)$/) {
		$newsetversions{$1}='mostrecent';
	    }
	}
    } elsif ($ENV{'form.setcurrent'}) {
	$haschanged=1;
	foreach (keys %hash) {
	    if ($_=~/^ids\_(\/res\/.+)$/) {
		my $getvers=&Apache::lonnet::getversion($1);
		if ($getvers>0) {
		    $newsetversions{$1}=$getvers;
		}
	    }
	}
    } elsif ($ENV{'form.setversions'}) {
	$haschanged=1;
	foreach (keys %ENV) {
	    if ($_=~/^form\.set_version_(.+)$/) {
		my $src=$1;
		if (($ENV{$_}) && ($ENV{$_} ne $setversions{$src})) {
		    $newsetversions{$src}=$ENV{$_};
		}
	    }
	}
    }
    if ($haschanged) {
        if (&Apache::lonnet::put('resourceversions',\%newsetversions,
			  $ENV{'course.'.$ENV{'request.course.id'}.'.domain'},
			  $ENV{'course.'.$ENV{'request.course.id'}.'.num'}) eq 'ok') {		
	    $r->print('<h1>'.&mt('Your Version Settings have been Stored').'</h1>');
	} else {
	    $r->print('<h1><font color="red">'.&mt('An Error Occured while Attempting to Store your Version Settings').'</font></h1>');
	}
	&mark_hash_old();
    }
    &changewarning($r,'');
    if ($ENV{'form.timerange'} eq 'all') {
# show all documents
	$header=&mt('All Documents in Course');
	$allsel=1;
	foreach (keys %hash) {
	    if ($_=~/^ids\_(\/res\/.+)$/) {
		my $src=$1;
		$changes{$src}=1;
	    }
	}
    } else {
# show documents which changed
	%changes=&Apache::lonnet::dump
	 ('versionupdate',$ENV{'course.'.$ENV{'request.course.id'}.'.domain'},
                     $ENV{'course.'.$ENV{'request.course.id'}.'.num'});
	my $firstkey=(keys %changes)[0];
	unless ($firstkey=~/^error\:/) {
	    unless ($ENV{'form.timerange'}) {
		$ENV{'form.timerange'}=604800;
	    }
	    my $seltext=&mt('during the last').' '.$ENV{'form.timerange'}.' '
		.&mt('seconds');
	    if ($ENV{'form.timerange'}==-1) {
		$seltext='since start of course';
		$startsel='selected';
		$ENV{'form.timerange'}=time;
	    }
	    $starttime=time-$ENV{'form.timerange'};
	    if ($ENV{'form.timerange'}==2592000) {
		$seltext=&mt('during the last month').' ('.&Apache::lonlocal::locallocaltime($starttime).')';
		$monthsel='selected';
	    } elsif ($ENV{'form.timerange'}==604800) {
		$seltext=&mt('during the last week').' ('.&Apache::lonlocal::locallocaltime($starttime).')';
		$weeksel='selected';
	    } elsif ($ENV{'form.timerange'}==86400) {
		$seltext=&mt('since yesterday').' ('.&Apache::lonlocal::locallocaltime($starttime).')';
		$daysel='selected';
	    }
	    $header=&mt('Content changed').' '.$seltext;
	} else {
	    $header=&mt('No content modifications yet.');
	}
    }
    %setversions=&Apache::lonnet::dump('resourceversions',
			  $ENV{'course.'.$ENV{'request.course.id'}.'.domain'},
			  $ENV{'course.'.$ENV{'request.course.id'}.'.num'});
    my %lt=&Apache::lonlocal::texthash
	      ('st' => 'Version changes since start of Course',
	       'lm' => 'Version changes since last Month',
	       'lw' => 'Version changes since last Week',
	       'sy' => 'Version changes since Yesterday',
               'al' => 'All Resources (possibly large output)',
	       'sd' => 'Display',
	       'fi' => 'File',
	       'md' => 'Modification Date',
               'mr' => 'Most recently published Version',
	       've' => 'Version used in Course',
               'vu' => 'Set Version to be used in Course',
'sv' => 'Set Versions to be used in Course according to Selections below',
'sm' => 'Keep all Resources up-to-date with most recent Versions (default)',
'sc' => 'Set all Resource Versions to current Version (Fix Versions)',
	       'di' => 'Differences');
    $r->print(<<ENDHEADERS);
<form action="/adm/coursedocs" method="post">
<input type="hidden" name="versions" value="1" />
<input type="submit" name="setmostrecent" value="$lt{'sm'}" />
<input type="submit" name="setcurrent" value="$lt{'sc'}" /><hr />
<select name="timerange">
<option value='all' $allsel>$lt{'al'}</option>
<option value="-1" $startsel>$lt{'st'}</option>
<option value="2592000" $monthsel>$lt{'lm'}</option>
<option value="604800" $weeksel>$lt{'lw'}</option>
<option value="86400" $daysel>$lt{'sy'}</option>
</select>
<input type="submit" name="display" value="$lt{'sd'}" />
<h3>$header</h3>
<input type="submit" name="setversions" value="$lt{'sv'}" />
<table border="0">
ENDHEADERS
    foreach (sort keys %changes) {
	if ($changes{$_}>$starttime) {
	    my ($root,$extension)=($_=~/^(.*)\.(\w+)$/);
	    my $currentversion=&Apache::lonnet::getversion($_);
	    if ($currentversion<0) {
		$currentversion=&mt('Could not be determined.');
	    }
	    my $linkurl=&Apache::lonnet::clutter($_);
	    $r->print(
		      '<tr><td colspan="5"><br /><br /><font size="+1"><b>'.
		      &Apache::lonnet::gettitle($linkurl).
                      '</b></font></td></tr>'.
                      '<tr><td>&nbsp;&nbsp;&nbsp;</td>'.
                      '<td colspan="4">'.
                      '<a href="'.$linkurl.'" target="cat">'.$linkurl.
		      '</a></td></tr>'.
                      '<tr><td></td>'.
                      '<td title="'.$lt{'md'}.'">'.
		      &Apache::lonlocal::locallocaltime(
                           &Apache::lonnet::metadata($root.'.'.$extension,
                                                     'lastrevisiondate')
                                                        ).
                      '</td>'.
                      '<td title="'.$lt{'mr'}.'"><nobr>Most Recent: '.
                      '<font size="+1">'.$currentversion.'</font>'.
                      '</nobr></td>'.
                      '<td title="'.$lt{'ve'}.'"><nobr>In Course: '.
                      '<font size="+1">');
# Used in course
	    my $usedversion=$hash{'version_'.$linkurl};
	    if (($usedversion) && ($usedversion ne 'mostrecent')) {
		$r->print($usedversion);
	    } else {
		$r->print($currentversion);
	    }
	    $r->print('</font></nobr></td><td title="'.$lt{'vu'}.'">'.
                      '<nobr>Use: ');
# Set version
	    $r->print(&Apache::loncommon::select_form($setversions{$linkurl},
						      'set_version_'.$linkurl,
						      ('select_form_order' =>
						       ['',1..$currentversion,'mostrecent'],
						       '' => '',
						       'mostrecent' => 'most recent',
						       map {$_,$_} (1..$currentversion))));
	    $r->print('</nobr></td></tr><tr><td></td>');
	    my $lastold=1;
	    for (my $prevvers=1;$prevvers<$currentversion;$prevvers++) {
		my $url=$root.'.'.$prevvers.'.'.$extension;
		if (&Apache::lonnet::metadata($url,'lastrevisiondate')<
		    $starttime) {
		    $lastold=$prevvers;
		}
	    }
            # 
            # Code to figure out how many version entries should go in
            # each of the four columns
            my $entries_per_col = 0;
            my $num_entries = ($currentversion-$lastold);
            if ($num_entries % 4 == 0) {
                $entries_per_col = $num_entries/4;
            } else {
                $entries_per_col = $num_entries/4 + 1;
            }
            my $entries_count = 0;
            $r->print('<td valign="top"><font size="-2">'); 
            my $cols_output = 1;
            for (my $prevvers=$lastold;$prevvers<$currentversion;$prevvers++) {
		my $url=$root.'.'.$prevvers.'.'.$extension;
		$r->print('<nobr><a href="'.&Apache::lonnet::clutter($url).
			  '">'.&mt('Version').' '.$prevvers.'</a> ('.
			  &Apache::lonlocal::locallocaltime(
                                &Apache::lonnet::metadata($url,
                                                          'lastrevisiondate')
                                                            ).
			  ')');
		if (&Apache::loncommon::fileembstyle($extension) eq 'ssi') {
                    $r->print(' <a href="/adm/diff?filename='.
			      &Apache::lonnet::clutter($root.'.'.$extension).
			      '&versionone='.$prevvers.
			      '">'.&mt('Diffs').'</a>');
		}
		$r->print('</nobr><br />');
                if (++$entries_count % $entries_per_col == 0) {
                    $r->print('</font></td>');
                    if ($cols_output != 4) {
                        $r->print('<td valign="top"><font size="-2">');
                        $cols_output++;
                    }
                }
	    }
            while($cols_output++ < 4) {
                $r->print('</font></td><td><font>')
            }
	    $r->print('</font></td></tr>'."\n");
	}
    }
    $r->print('</table></form>');
    $r->print('<h1>'.&mt('Done').'.</h1>');

    &untiehash();
}

sub mark_hash_old {
    my $retie_hash=0;
    &Apache::lonnet::logthis(" markold1 -- $hashtied $retie_hash ");
    if ($hashtied) {
	$retie_hash=1;
	&Apache::lonnet::logthis(" markold2 -- $hashtied $retie_hash ");
	&untiehash();
	&Apache::lonnet::logthis(" markold3 -- $hashtied $retie_hash ");
    }
    &tiehash('write');
    &Apache::lonnet::logthis(" markold4 -- $hashtied $retie_hash ");
    $hash{'old'}=1;
    &Apache::lonnet::logthis(" markold5 -- $hashtied $retie_hash ");    
    &untiehash();
    &Apache::lonnet::logthis(" markold6 -- $hashtied $retie_hash ");
    if ($retie_hash) { &tiehash(); }
    &Apache::lonnet::logthis(" markold7 -- $hashtied $retie_hash ");
}

sub is_hash_old {
    my $untie_hash=0;
    if (!$hashtied) {
	$untie_hash=1;
	&tiehash();
    }
    my $return=$hash{'old'};
    if ($untie_hash) { &untiehash(); }
    return $return;
}

sub changewarning {
    my ($r,$postexec)=@_;
    if (!&is_hash_old()) { return; }
    $r->print(
'<script>function reinit(tf) { tf.submit();'.$postexec.' }</script>'. 
'<form method="post" action="/adm/roles" target="loncapaclient">'.
'<input type="hidden" name="orgurl" value="/adm/coursedocs?folderpath='.
&Apache::lonnet::escape($ENV{'form.folderpath'}).
'" /><input type="hidden" name="selectrole" value="1" /><h3><font color="red">'.
&mt('Changes will become active for your current session after').
' <input type="hidden" name="'.
$ENV{'request.role'}.'" value="1" /><input type="button" value="'.
&mt('re-initializing course').'" onClick="reinit(this.form)"/>'.&mt(', or the next time you log in.').
$help{'Caching'}.'</font></h3></form>');
}

# ================================================================ Main Handler
sub handler {
    my $r = shift;
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    return OK if $r->header_only;

# --------------------------------------------- Initialize help topics for this
  foreach ('Adding_Course_Doc','Main_Course_Documents',
           'Adding_External_Resource','Navigate_Content',
           'Adding_Folders','Docs_Overview', 'Load_Map',
           'Supplemental','Score_Upload_Form','Adding_Pages',
           'Importing_LON-CAPA_Resource','Uploading_From_Harddrive',
	   'Check_Resource_Versions','Verify_Content') {
      $help{$_}=&Apache::loncommon::help_open_topic('Docs_'.$_);
  }
    # Composite help files
    $help{'Syllabus'} = &Apache::loncommon::help_open_topic(
		    'Docs_About_Syllabus,Docs_Editing_Templated_Pages');
    $help{'Simple Page'} = &Apache::loncommon::help_open_topic(
		    'Docs_About_Simple_Page,Docs_Editing_Templated_Pages');
    $help{'Simple Problem'} = &Apache::loncommon::help_open_topic(
		    'Option_Response_Simple');
    $help{'Bulletin Board'} = &Apache::loncommon::help_open_topic(
		    'Docs_About_Bulletin_Board,Docs_Editing_Templated_Pages');
    $help{'My Personal Info'} = &Apache::loncommon::help_open_topic(
		  'Docs_About_My_Personal_Info,Docs_Editing_Templated_Pages');
    $help{'Caching'} = &Apache::loncommon::help_open_topic('Caching');

  if ($ENV{'form.verify'}) {
      &verifycontent($r);
  } elsif ($ENV{'form.versions'}) {
      &checkversions($r);
  } elsif ($ENV{'form.dumpcourse'}) {
      &dumpcourse($r);
  } elsif ($ENV{'form.exportcourse'}) {
      &exportcourse($r);
  } else {
# is this a standard course?

    my $standard=($ENV{'request.course.uri'}=~/^\/uploaded\//);
    my $forcestandard;
    my $forcesupplement;
    my $script='';
    my $allowed;
    my $events='';
    my $showdoc=0;
    my $containertag;
    my $uploadtag;
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
					    ['folderpath','pagepath']);
    if ($ENV{'form.folderpath'}) {
	my (@folderpath)=split('&',$ENV{'form.folderpath'});
	$ENV{'form.foldername'}=&Apache::lonnet::unescape(pop(@folderpath));
	$ENV{'form.folder'}=pop(@folderpath);
    }
    if ($ENV{'form.pagepath'}) {
        my (@pagepath)=split('&',$ENV{'form.pagepath'});
        $ENV{'form.pagename'}=&Apache::lonnet::unescape(pop(@pagepath));
        $ENV{'form.folder'}=pop(@pagepath);
        $containertag = '<input type="hidden" name="pagepath" value="" />';
        $uploadtag = '<input type="hidden" name="pagepath" value="'.$ENV{'form.pagepath'}.'" />';
    }
    if ($r->uri=~/^\/adm\/coursedocs\/showdoc\/(.*)$/) {
       $showdoc='/'.$1;
    }
    unless ($showdoc) { # got called from remote
       $forcestandard=($ENV{'form.folder'}=~/^default_/);
       $forcesupplement=($ENV{'form.folder'}=~/^supplemental_/);

# does this user have privileges to post, etc?
       $allowed=&Apache::lonnet::allowed('mdc',$ENV{'request.course.id'});
       if ($allowed) { 
         &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['cmd']);
         $script=&Apache::lonratedt::editscript('simple'); 
       }
    } else { # got called in sequence from course
       $allowed=0;
       $script='</script>'.&Apache::lonmenu::registerurl(1,undef).'<script>';
       $events='onLoad="'.&Apache::lonmenu::loadevents.
           '" onUnload="'.&Apache::lonmenu::unloadevents.'"';
    }

# get course data
    my $coursenum=$ENV{'course.'.$ENV{'request.course.id'}.'.num'};
    my $coursedom=$ENV{'course.'.$ENV{'request.course.id'}.'.domain'};

# get personal data
 
    my $uname=$ENV{'user.name'};
    my $udom=$ENV{'user.domain'};
    my $plainname=&Apache::lonnet::escape(
                     &Apache::loncommon::plainname($uname,$udom));

# graphics settings

    $iconpath = $r->dir_config('lonIconsURL') . "/";

    my $now=time;

# print screen
    $r->print(<<ENDDOCUMENT);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
<script>
$script
</script>
ENDDOCUMENT
   if ($allowed) {
    $r->print(<<ENDNEWSCRIPT);
<script>
function makenewfolder(targetform,folderseq) {
    var foldername=prompt('Name of New Folder','New Folder');
    if (foldername) {
	targetform.importdetail.value=foldername+"="+folderseq;
        targetform.submit();
    }
}

function makenewpage(targetform,folderseq) {
    var pagename=prompt('Name of New Page','New Page');
    if (pagename) {
        targetform.importdetail.value=pagename+"="+folderseq;
        targetform.submit();
    }
}

function makenewext(targetname) {
    this.document.forms.extimport.useform.value=targetname;
    window.open('/adm/rat/extpickframe.html');
}

function makeexamupload() {
   var title=prompt('Listed Title for the Uploaded Score');
   if (title) { 
    this.document.forms.newexamupload.importdetail.value=
	title+'=/res/lib/templates/examupload.problem';
    this.document.forms.newexamupload.submit();
   }
}

function makesmppage() {
   var title=prompt('Listed Title for the Page');
   if (title) { 
    this.document.forms.newsmppg.importdetail.value=
	title+'=/adm/$udom/$uname/$now/smppg';
    this.document.forms.newsmppg.submit();
   }
}

function makesmpproblem() {
   var title=prompt('Listed Title for the Problem');
   if (title) { 
    this.document.forms.newsmpproblem.importdetail.value=
	title+'=/res/lib/templates/simpleproblem.problem';
    this.document.forms.newsmpproblem.submit();
   }
}

function makebulboard() {
   var title=prompt('Listed Title for the Bulletin Board');
   if (title) {
    this.document.forms.newbul.importdetail.value=
	title+'=/adm/$udom/$uname/$now/bulletinboard';
    this.document.forms.newbul.submit();
   }
}

function makeabout() {
   var user=prompt("Enter user\@domain for User's 'About Me' Page");
   if (user) {
       var comp=new Array();
       comp=user.split('\@');
       if ((typeof(comp[0])!=undefined) && (typeof(comp[1])!=undefined)) {
	   if ((comp[0]) && (comp[1])) {
	       this.document.forms.newaboutsomeone.importdetail.value=
		   'About '+user+'=/adm/'+comp[1]+'/'+comp[0]+'/aboutme';
	       this.document.forms.newaboutsomeone.submit();
	   }
       }
   }
}

function makeims() {
    var caller = document.forms.ims.folder.value
    var newlocation = "/adm/imsimportdocs?folder="+caller+"&phase=one"
    newWindow = window.open("","IMSimport","HEIGHT=700,WIDTH=750,scrollbars=yes")
    newWindow.location.href = newlocation
}


function finishpick() {
    var title=this.document.forms.extimport.title.value;
    var url=this.document.forms.extimport.url.value;
    var form=this.document.forms.extimport.useform.value;
    eval
     ('this.document.forms.'+form+'.importdetail.value="'+title+'='+url+
    '";this.document.forms.'+form+'.submit();');
}

function changename(folderpath,index,oldtitle,container) {
    var title=prompt('New Title',oldtitle);
    if (title) {
	this.document.forms.renameform.title.value=title;
	this.document.forms.renameform.cmd.value='rename_'+index;
        if (container == 'sequence') {
	    this.document.forms.renameform.folderpath.value=folderpath;
        }
        if (container == 'page') {
            this.document.forms.renameform.pagepath.value=folderpath;
        }
        this.document.forms.renameform.submit();
    }
}

function removeres(folderpath,index,oldtitle,container) {
    if (confirm('Remove "'+oldtitle+'"?')) {
	this.document.forms.renameform.cmd.value='del_'+index;
        if (container == 'sequence') {
            this.document.forms.renameform.folderpath.value=folderpath;
        }
        if (container == 'page') {
            this.document.forms.renameform.pagepath.value=folderpath;
        }
        this.document.forms.renameform.submit();
    }
}

</script>

ENDNEWSCRIPT
  }
# -------------------------------------------------------------------- Body tag
  $r->print('</head>'.
            &Apache::loncommon::bodytag('Course Documents','',$events,
					'','',$showdoc).
	    &Apache::loncommon::help_open_menu('','','','',273,'RAT'));
  unless ($showdoc) {
# -----------------------------------------------------------------------------
       my %lt=&Apache::lonlocal::texthash(
                'uplm' => 'Upload a new main course document',
                'upls' => 'Upload a new supplemental course document',
                'impp' => 'Import a published document',
                'spec' => 'Special documents',
                'upld' => 'Upload Document',
                'srch' => 'Search',
                'impo' => 'Import',
                'selm' => 'Select Map',
                'load' => 'Load Map',
                'newf' => 'New Folder',
                'newp' => 'New Composite Page',
                'extr' => 'External Resource',
                'syll' => 'Syllabus',
                'navc' => 'Navigate Contents',
                'sipa' => 'Simple Page',
                'sipr' => 'Simple Problem',
                'scuf' => 'Score Upload Form',
                'bull' => 'Bulletin Board',
                'mypi' => 'My Personal Info',
		'abou' => 'About User',
                'imsf' => 'Import IMS package',
                'file' =>  'File',
                'title' => 'Title',
                'comment' => 'Comment' 
					  );
# -----------------------------------------------------------------------------
    if ($allowed) {
       my $dumpbut=&dumpbutton();
       my $exportbut=&exportbutton();
       my %lt=&Apache::lonlocal::texthash(
					 'vc' => 'Verify Content',
					 'cv' => 'Check/Set Resource Versions',
					  );

       my $folderpath=$ENV{'form.folderpath'};
       if (!$folderpath) {
	   if ($ENV{'form.folder'} eq '' ||
	       $ENV{'form.folder'} eq 'supplemental') {
	       $folderpath='default&'.
		   &Apache::lonnet::escape(&mt('Main Course Documents'));
	   }
       }
       unless ($ENV{'form.pagepath'}) {
           $containertag = '<input type="hidden" name="folderpath" value="" />';
           $uploadtag = '<input type="hidden" name="folderpath" value="'.$folderpath.'" />';
       }

       $r->print(<<ENDCOURSEVERIFY);
<form name="renameform" method="post" action="/adm/coursedocs">
<input type="hidden" name="title" />
<input type="hidden" name="cmd" />
$containertag
</form>
<form name="simpleedit" method="post" action="/adm/coursedocs">
<input type=hidden name="importdetail" value="">
$uploadtag
</form>
<form action="/adm/coursedocs" method="post" name="courseverify">
<table bgcolor="#AAAAAA" width="100%" cellspacing="4" cellpadding="4">
<tr><td bgcolor="#DDDDCC">
<input type="submit" name="verify" value="$lt{'vc'}" />$help{'Verify_Content'}
</td><td bgcolor="#DDDDCC">
    <input type="submit" name="versions" value="$lt{'cv'}" />$help{'Check_Resource_Versions'}
$dumpbut
$exportbut
</td></tr></table>
</form>
ENDCOURSEVERIFY
       $r->print(&Apache::loncommon::help_open_topic('Docs_Adding_Course_Doc',
		     &mt('Editing the Table of Contents for your Course')));
    }
# --------------------------------------------------------- Standard documents
    $r->print('<table border=2 cellspacing=4 cellpadding=4>');
    if (($standard) && ($allowed) && (!$forcesupplement)) {
	$r->print('<tr><td bgcolor="#BBBBBB">');
#  '<h2>'.&mt('Main Course Documents').
#  ($allowed?' '.$help{'Main_Course_Documents'}:'').'</h2>');
       my $folder=$ENV{'form.folder'};
       if ($folder eq '' || $folder eq 'supplemental') {
           $folder='default';
	   $ENV{'form.folderpath'}='default&'.&Apache::lonnet::escape(&mt('Main Course Documents'));
       }
       my $postexec='';
       if ($folder eq 'default') {
	   $r->print('<script>this.window.name="loncapaclient";</script>');
       } else {
           #$postexec='self.close();';
       }
       $hadchanges=0;
       &editor($r,$coursenum,$coursedom,$folder,$allowed);
       if ($hadchanges) {
	   &mark_hash_old()
       }
       &changewarning($r,$postexec);
       my $folderseq='/uploaded/'.$coursedom.'/'.$coursenum.'/default_'.time.
                     '.sequence';
       my $pageseq = '/uploaded/'.$coursedom.'/'.$coursenum.'/default_'.time.
                     '.page';

       $r->print(<<ENDFORM);
<table cellspacing=4 cellpadding=4><tr>
<th bgcolor="#DDDDDD">$lt{'uplm'}</th>
<th bgcolor="#DDDDDD">$lt{'impp'}</th>
<th bgcolor="#DDDDDD">$lt{'spec'}</th>
</tr>
<tr><td bgcolor="#DDDDDD">
$lt{'file'}:<br />
<form action="/adm/coursedocs" method="post" enctype="multipart/form-data">
<input type="file" name="uploaddoc" size="40">
<br />
$lt{'title'}:<br />
<input type="text" size="50" name="comment">
$uploadtag
<input type="hidden" name="cmd" value="upload_default">
<nobr>
<input type="submit" value="$lt{'upld'}">
 $help{'Uploading_From_Harddrive'}
</nobr>
</form>
</td>
<td bgcolor="#DDDDDD">
<form action="/adm/coursedocs" method="post" name="simpleeditdefault">
$uploadtag
<input type=button onClick="javascript:groupsearch()" value="$lt{'srch'}">
<nobr>
<input type=button onClick="javascript:groupimport();" value="$lt{'impo'}">
$help{'Importing_LON-CAPA_Resource'}
</nobr>
<p>
<hr />
<input type="text" size="20" name="importmap"><br />
<nobr><input type=button 
onClick="javascript:openbrowser('simpleeditdefault','importmap','sequence,page','')"
value="$lt{'selm'}"> <input type="submit" name="loadmap" value="$lt{'load'}">
$help{'Load_Map'}</nobr>
</p>
</form>
</td>
<td bgcolor="#DDDDDD">
ENDFORM
       unless ($ENV{'form.pagepath'}) {
           $r->print(<<ENDFORM);
<form action="/adm/coursedocs" method="post" name="newfolder">
<input type="hidden" name="folderpath" value="$ENV{'form.folderpath'}" />
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newfolder" type="button"
onClick="javascript:makenewfolder(this.form,'$folderseq');"
value="$lt{'newf'}" />$help{'Adding_Folders'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newpage">
<input type="hidden" name="folderpath" value="$ENV{'form.folderpath'}" />
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newpage" type="button"
onClick="javascript:makenewpage(this.form,'$pageseq');"
value="$lt{'newp'}" />$help{'Adding_Pages'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newext">
$uploadtag
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newext" type="button" onClick="javascript:makenewext('newext');"
value="$lt{'extr'}" /> $help{'Adding_External_Resource'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newsyl">
$uploadtag
<input type=hidden name="importdetail" 
value="Syllabus=/public/$coursedom/$coursenum/syllabus">
<nobr>
<input name="newsyl" type="submit" value="$lt{'syll'}" /> 
 $help{'Syllabus'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newnav">
$uploadtag
<input type=hidden name="importdetail" 
value="Navigate Content=/adm/navmaps">
<nobr>
<input name="newnav" type="submit" value="$lt{'navc'}" />
$help{'Navigate_Content'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newsmppg">
$uploadtag
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newsmppg" type="button" value="$lt{'sipa'}"
onClick="javascript:makesmppage();" /> $help{'Simple Page'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newsmpproblem">
$uploadtag
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newsmpproblem" type="button" value="$lt{'sipr'}"
onClick="javascript:makesmpproblem();" />$help{'Simple Problem'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newexamupload">
$uploadtag
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newexamupload" type="button" value="$lt{'scuf'}"
onClick="javascript:makeexamupload();" />
$help{'Score_Upload_Form'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newbul">
$uploadtag
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newbulletin" type="button" value="$lt{'bull'}"
onClick="javascript:makebulboard();" />
$help{'Bulletin Board'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newaboutme">
$uploadtag
<input type=hidden name="importdetail" 
value="$plainname=/adm/$udom/$uname/aboutme">
<nobr>
<input name="newaboutme" type="submit" value="$lt{'mypi'}" />
$help{'My Personal Info'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newaboutsomeone">
$uploadtag
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newaboutsomeone" type="button" value="$lt{'abou'}" 
onClick="javascript:makeabout();" />
</nobr>
ENDFORM
       }
       if ($ENV{'form.pagepath'}) {
           $r->print(<<ENDBLOCK);
<form action="/adm/coursedocs" method="post" name="newsmpproblem">
$uploadtag
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newsmpproblem" type="button" value="$lt{'sipr'}"
onClick="javascript:makesmpproblem();" />$help{'Simple Problem'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="newexamupload">
$uploadtag
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newexamupload" type="button" value="$lt{'scuf'}"
onClick="javascript:makeexamupload();" />
$help{'Score_Upload_Form'}
</nobr>
</form>
ENDBLOCK
       } else {
           $r->print(<<ENDFORM);
</form>
<form action="/adm/imsimportdocs" method="post" name="ims">
<input type="hidden" name="folder" value="$folder" />
<input name="imsimport" type="button" value="$lt{'imsf'}" onClick="javascript:makeims();" />
</nobr>
</form>
ENDFORM
       }
       $r->print('</td></tr>'."\n".
'</table>');
       $r->print('</td></tr>');
    }
# ----------------------------------------------------- Supplemental documents
    if (!$forcestandard) {
       $r->print('<tr><td bgcolor="#BBBBBB">');
# '<h2>'.&mt('Supplemental Course Documents').
#  ($allowed?' '.$help{'Supplemental'}:'').'</h2>');
       my $folder=$ENV{'form.folder'};
       unless ($folder=~/^supplemental/) {
	   $folder='supplemental';
       }
       if ($folder =~ /^supplemental$/ &&
	   $ENV{'form.folderpath'} =~ /^default\&/) {
	   $ENV{'form.folderpath'}='supplemental&'.
	       &Apache::lonnet::escape(&mt('Supplemental Course Documents'));
       }
       &editor($r,$coursenum,$coursedom,$folder,$allowed);
       if ($allowed) {
       my $folderseq=
                  '/uploaded/'.$coursedom.'/'.$coursenum.'/supplemental_'.time.
                     '.sequence';

          $r->print(<<ENDSUPFORM);
<table cellspacing=4 cellpadding=4><tr>
<th bgcolor="#DDDDDD">$lt{'upls'}</th>
<th bgcolor="#DDDDDD">$lt{'spec'}</th>
</tr>
<tr><td bgcolor="#DDDDDD">
<form action="/adm/coursedocs" method="post" enctype="multipart/form-data">
<input type="file" name="uploaddoc" size="40">
<br />$lt{'comment'}:<br />
<textarea cols=50 rows=4 name='comment'>
</textarea>
<br />
<input type="hidden" name="folderpath" value="$ENV{'form.folderpath'}" />
<input type="hidden" name="cmd" value="upload_supplemental">
<nobr>
<input type="submit" value="$lt{'upld'}">
 $help{'Uploading_From_Harddrive'}
</nobr>
</form>
</td>
<td bgcolor="#DDDDDD">
<form action="/adm/coursedocs" method="post" name="supnewfolder">
<input type="hidden" name="folderpath" value="$ENV{'form.folderpath'}" />
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newfolder" type="button"
onClick="javascript:makenewfolder(this.form,'$folderseq');"
value="$lt{'newf'}" /> $help{'Adding_Folders'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="supnewext">
<input type="hidden" name="folderpath" value="$ENV{'form.folderpath'}" />
<input type=hidden name="importdetail" value="">
<nobr>
<input name="newext" type="button" 
onClick="javascript:makenewext('supnewext');"
value="$lt{'extr'}" /> $help{'Adding_External_Resource'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="supnewsyl">
<input type="hidden" name="folderpath" value="$ENV{'form.folderpath'}" />
<input type=hidden name="importdetail" 
value="Syllabus=/public/$coursedom/$coursenum/syllabus">
<nobr>
<input name="newsyl" type="submit" value="$lt{'syll'}" />
$help{'Syllabus'}
</nobr>
</form>
<form action="/adm/coursedocs" method="post" name="subnewaboutme">
<input type="hidden" name="folderpath" value="$ENV{'form.folderpath'}" />
<input type=hidden name="importdetail" 
value="$plainname=/adm/$udom/$uname/aboutme">
<nobr>
<input name="newaboutme" type="submit" value="$lt{'mypi'}" />
$help{'My Personal Info'}
</nobr>
</form>
</td></tr>
</table></td></tr>
ENDSUPFORM
       }
    }
    if ($allowed) {
	$r->print('<form name="extimport"><input type="hidden" name="title"><input type="hidden" name="url"><input type="hidden" name="useform"></form>');
    }
    $r->print('</table>');
  } else {
# -------------------------------------------------------- This is showdoc mode
      $r->print("<h1>".&mt('Uploaded Document').' - '.
		&Apache::lonnet::gettitle($r->uri).'</h1><p>'.
&mt('It is recommended that you use an up-to-date virus scanner before handling this file.')."</p><p><table>".
         &entryline(0,&mt("Click to download or use your browser's Save Link function"),$showdoc).'</table></p>');
  }
 }
 $r->print('</body></html>');
 return OK;
} 

1;
__END__
