# The LearningOnline Network with CAPA
#
# $Id: lonstudentsubmissions.pm,v 1.28 2004/11/19 19:44:39 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
package Apache::lonstudentsubmissions;

use strict;
use Apache::lonnet();
use Apache::loncommon();
use Apache::lonhtmlcommon();
use Apache::loncoursedata();
use Apache::lonstatistics;
use Apache::lonlocal;
use Apache::lonstathelpers;
use HTML::Entities();
use Time::Local();
use Spreadsheet::WriteExcel();

my @SubmitButtons = ({ name => 'SelectAnother',
                       text => 'Choose a different Problem' },
                     { name => 'Generate',
                       text => 'Generate Report'},
                     );

sub BuildStudentSubmissionsPage {
    my ($r,$c)=@_;
    #
    my %Saveable_Parameters = ('Status' => 'scalar',
                               'Section' => 'array',
                               'NumPlots' => 'scalar',
                               );
    &Apache::loncommon::store_course_settings('student_submissions',
                                              \%Saveable_Parameters);
    &Apache::loncommon::restore_course_settings('student_submissions',
                                                \%Saveable_Parameters);
    #
    &Apache::lonstatistics::PrepareClasslist();
    #
    $r->print(&CreateInterface());
    #
    my @Students = @Apache::lonstatistics::Students;
    #
    if (@Students < 1) {
        $r->print('<h2>There are no students in the sections selected</h2>');
    }
    #
    my @CacheButtonHTML = 
        &Apache::lonstathelpers::manage_caches($r,'Statistics','stats_status',
                                   '<h3>'.&mt('Loading student data').'</h3>');
    $r->rflush();
    #
    if (exists($ENV{'form.problemchoice'}) && 
        ! exists($ENV{'form.SelectAnother'})) {
        foreach my $button (@SubmitButtons) {
            if ($button->{'name'} eq 'break') {
                $r->print("<br />\n");
            } else {
                $r->print('<input type="submit" name="'.$button->{'name'}.'" '.
                          'value="'.&mt($button->{'text'}).'" />');
                $r->print('&nbsp;'x5);
            }
        }
        foreach my $html (@CacheButtonHTML) {
            $r->print($html.('&nbsp;'x5));
        }
        #
        $r->print('<hr />'.$/);
        $r->rflush();
        #
        # Determine which problems we are to analyze
        my @Symbs = 
            &Apache::lonstathelpers::get_selected_symbs('problemchoice');
        foreach my $selected (@Symbs) {
            $r->print('<input type="hidden" name="problemchoice" value="'.
                      $selected.'" />'.$/);
        }
        #
        # Get resource objects
        my $navmap = Apache::lonnavmaps::navmap->new();
        if (!defined($navmap)) {
            $r->print('<h1>'.&mt("Internal error").'</h1>');
            return;
        }
        my %already_seen;
        my @Problems;
        foreach my $symb (@Symbs) {
            my $resource = $navmap->getBySymb($symb);
            push(@Problems,$resource);
        }
        # 
        if (! scalar(@Problems) || ! defined($Problems[0])) {
            $r->print('resource is undefined');
        } else {
            if (scalar(@Problems) == 1) {
                my $resource = $Problems[0];
                $r->print('<h1>'.$resource->title.'</h1>');
                $r->print('<h3>'.$resource->src.'</h3>');
                if ($ENV{'form.renderprob'} eq 'true') {
                    $r->print(
                              &Apache::lonstathelpers::render_resource({src => $resource->src})
                              );
                    $r->rflush();
                }
            }
            if ($ENV{'form.output'} eq 'excel') {
                &prepare_excel_output($r,\@Problems,\@Students);
            } elsif ($ENV{'form.output'} eq 'csv') {
                &prepare_csv_output($r,\@Problems,\@Students);
            } else {
                &prepare_html_output($r,\@Problems,\@Students);
            }
        }
        $r->print('<hr />');
    } else {
        $r->print('<input type="submit" name="Generate" value="'.
                  &mt('Prepare Report').'" />');
        $r->print('&nbsp;'x5);
        $r->print('<p>'.
                  &mt('Computing correct answers greatly increasese the amount of time required to prepare a report.').
                  '</p>');
        $r->print('<p>'.
                  &mt('please select problems and use the <b>Prepare Report</b> button to continue.').
                  '</p>');
        $r->print(&Apache::lonstathelpers::MultipleProblemSelector
                  (undef,'problemchoice','Statistics'));
    }
}

#########################################################
#########################################################
##
##    HTML Output Routines
##
#########################################################
#########################################################
sub prepare_html_output {
    my ($r,$problems,$students) = @_;
    my $c = $r->connection();
    #
    # Set a flag for the case when there is just one problem
    my $single_response = 0;
    if (scalar(@$problems) == 1 &&
        $problems->[0]->countResponses == 1) {
        $single_response = 1;
    }
    #
    # Compute the number of columns per response
    my @response_headers = ('Submission');
    if ($ENV{'form.correctans'} eq 'true') {
        push(@response_headers,'Correct');
    } 
    if ($ENV{'form.prob_status'} eq 'true') {
        push(@response_headers,'Award Detail'); 
        push(@response_headers,'Time');
        push(@response_headers,'Attempt');
        push(@response_headers,'Awarded');
    }
    my $response_multiplier = scalar(@response_headers);
    #
    # Create the table header
    my @student_columns = ('username','domain','id');
    #
    my %headers;
    my $student_column_count = scalar(@student_columns);
    $headers{'problem'} = qq{<th colspan="$student_column_count">\&nbsp;</th>};
    foreach (@student_columns) {
        $headers{'student'}.= '<th>'.ucfirst($_).'</th>';
    }
    #
    # we put the headers into the %headers hash
    my $total_col = scalar(@student_columns);
    my $nonempty_part_headers = 0;
    foreach my $prob (@$problems) {
        my $prob_span = 0;
        my $single_part = 0;
        if (scalar(@{$prob->parts}) == 1) {
            $single_part = 1;
        }
        foreach my $partid (@{$prob->parts}) {
            my $part_span = 0;
            my $responses = [$prob->responseIds($partid)];
            my $resptypes = [$prob->responseType($partid)];
            for (my $i=0;$i<scalar(@$responses);$i++) {
                if ($resptypes->[$i] eq 'essay') {
                    # do something clever, like shut up.
                } else {
                    $total_col += scalar(@response_headers);
                    $headers{'response'} .=
                        '<th colspan="'.scalar(@response_headers).'">'.
                        &mt('Response [_1]',$responses->[$i]).'</th>';
                    $headers{'student'}.= '<th>'.join('</th><th>',
                                                      @response_headers).
                                                          '</th>';
                    $part_span += scalar(@response_headers);
                }
            }
            if (! $single_part) {
                my $tmpname = $partid;
                if ($partid =~/^\d+$/) {
                    $tmpname = $prob->part_display($partid);
                }
                $headers{'part'} .= qq{<th colspan="$part_span">$tmpname</th>};
                $nonempty_part_headers = 1;
            } else {
                $headers{'part'} .= qq{<th colspan="$part_span">&nbsp</th>};
            }
            $prob_span += $part_span;
        }
        my $title = $prob->compTitle;
        if ($prob_span > 0) {
            $headers{'problem'}.= qq{<th colspan="$prob_span">$title</th>};
        } elsif ($single_response) {
            $prob_span = scalar(@student_columns);
            $headers{'problem'} = qq{<th colspan="$prob_span">$title</th>};
        }
    }
    if (exists($headers{'part'})) {
        $headers{'part'} = qq{<th colspan="$student_column_count">\&nbsp;</th>}.
            $headers{'part'};
    }
    if (exists($headers{'response'})) {
        $headers{'response'}=
            qq{<th colspan="$student_column_count">\&nbsp;</th>}.
            $headers{'response'};
    }
    my $full_header = $/.'<table>'.$/;
    $full_header .= '<tr align="left">'.$headers{'problem'}.'</tr>'.$/;
    if ($nonempty_part_headers) {
        $full_header .= '<tr align="left">'.$headers{'part'}.'</tr>'.$/;
    }
    $full_header .= '<tr align="left">'.$headers{'response'}.'</tr>'.$/;
    $full_header .= '<tr align="left">'.$headers{'student'}.'</tr>'.$/;
    #
    # Main loop
    my $count;
    $r->print($/.$full_header.$/);
    my $row_class = 'odd';   # css 
    foreach my $student (@$students) {
        my $student_row_data;
        if ($count++ >= 30) {
            $r->print('</table>'.$/.$full_header.$/);
            $count = 0;
        }
        last if ($c->aborted());
        foreach my $field (@student_columns) {
            $student_row_data .= 
                '<td valign="top">'.$student->{$field}.'</td>';
        }
        #
        # Figure out what it is we need to output for this student
        my @essays;
        my %problem_data;
        my $maxrow;
        foreach my $prob (@$problems) {
            my $symb = $prob->symb;
            $problem_data{$symb}={};
            foreach my $partid (@{$prob->parts}) {
                my @responses = $prob->responseIds($partid);
                my @response_type = $prob->responseType($partid);
                for (my $i=0;$i<=$#responses;$i++) {
                    my $respid   = $responses[$i];
                    my $resptype = $response_type[$i];
                    my $width = scalar(@response_headers);
                    $problem_data{$symb}->{$partid}->{$respid}={};
                    my $resp_data = $problem_data{$symb}->{$partid}->{$respid};
                    $resp_data->{'fake'} = qq{<td colspan="$width">&nbsp</td>};
                    my $results = 
                        &Apache::loncoursedata::get_response_data_by_student
                        ($student,$prob->symb(),$respid);
                    if (! defined($results)) {
                        $results = [];
                    }
                    #
                    if (scalar(@$results) > $maxrow && $resptype ne 'essay') {
                        $maxrow = scalar(@$results);
                    }
                    for (my $j=scalar(@$results)-1;$j>=0;$j--) {
                        my $response = $results->[$j];
                        if ($ENV{'form.all_sub'} ne 'true') {
                            next if ($j ne scalar(@$results)-1);
                        }
                        if ($resptype eq 'essay') {
                            push(@essays,
                                 &html_essay_results($student,
                                                     $prob,$partid,$respid,
                                                     $response,
                                                     $single_response).
                                 '</td>');
                        } else {
                            push(@{$resp_data->{'real'}},
                                 &html_results($student,
                                               $prob,$partid,$respid,
                                               $response,$resptype));
                        }
                    } 
                } # end of $i loop
            } # end of partid loop
        } # end of prob loop
        #
        # if there is no data, skip this student.
        next if (! $maxrow && ! scalar(@essays));
        #
        # Go through the problem data and output a row.
        if ($row_class eq 'even') {
            $row_class = 'odd'; 
        } else {
            $row_class = 'even'; 
        }
        my $printed_something;
        for (my $rows_output = 0;$rows_output<$maxrow;$rows_output++) {
            my $html;
            my $no_data = 1;
            foreach my $prob (@$problems) {
                my $symb = $prob->symb;
                foreach my $partid (@{$prob->parts}) {
                    my @responses     = $prob->responseIds($partid);
                    my @response_type = $prob->responseType($partid);
                    for (my $i=0;$i<=$#responses;$i++) {
                        my $respid   = $responses[$i];
                        my $resp_data = 
                            $problem_data{$symb}->{$partid}->{$respid};
                        next if ($response_type[$i] eq 'essay');
                        if (defined($resp_data->{'real'}->[$rows_output])) {
                            $html .= $resp_data->{'real'}->[$rows_output];
                            $no_data = 0;
                        } else {
                            $html .= $resp_data->{'fake'};
                        }
                    }
                }
            }
            if (! $no_data) {
                $r->print(qq{<tr class="$row_class">$student_row_data$html</tr>}.$/);
                $printed_something=1;
            }
        }
        if (@essays) {
            my $tr = qq{<tr class="$row_class">};
            my $td = qq{<td  valign="top" class="essay" colspan="$total_col">};
            if (! $printed_something) {
                $r->print($tr.$student_row_data.'</tr>'.$/);
            }
            $r->print($tr.$td.
                      join('</td></tr>'.$/.$tr.$td,@essays).'</td></tr>'.$/);
            undef(@essays);
        }
    } # end of student loop
    return;
}

#####################################################
##
##     HTML helper routines
##
#####################################################
sub html_essay_results {
    my ($student,$prob,$partid,$respid,$response,$single_response)=@_;
    #
    my $submission =$response->[&Apache::loncoursedata::RDs_submission()];
    $submission = &html_format_sub($submission,'essay');
    #
    my $correct = '';
    if ($ENV{'form.correctans'} eq 'true') {
        $correct = &Apache::lonstathelpers::get_student_answer
            ($prob,$student->{'username'},$student->{'domain'},
             $partid,$respid);
        $correct = &html_format_sub($correct,'essay');
    }
    my $Str;
    if (! $single_response) {
        my $id = $prob->compTitle;
        if (defined($partid) && $partid ne '0') {
            $id .= ' '.$prob->part_display($partid);
        }
        if (defined($respid)) {
            $id .= ' '.$respid;
        }
        $Str .= '<nobr>'.$id.'</nobr>'.('&nbsp;'x4);
    }
    if ($ENV{'form.prob_status'} eq 'true') {
        $Str .= '<nobr>';
        $Str .= $response->[&Apache::loncoursedata::RDs_awarddetail()].
            ('&nbsp;'x4);
        $Str .= &mt('Attempt [_1]',
                    $response->[&Apache::loncoursedata::RDs_tries()]).
                        ('&nbsp;'x4);
        $Str .= &Apache::lonlocal::locallocaltime
            ($response->[&Apache::loncoursedata::RDs_timestamp()]).
            ('&nbsp;'x4);
        $Str .= &mt('Awarded: [_1]',
                    $response->[&Apache::loncoursedata::RDs_awarded()]).
                        ('&nbsp;'x4);
        $Str .= '</nobr><br />';
    }
    $Str .= $submission;
    if (defined($correct) && $correct !~ /^\s*$/) {
        $Str .= '<hr /><b>'.&mt('Correct').'</b>'.$correct
    }
    return $Str;
}

sub html_results {
    my ($student,$prob,$partid,$respid,$response,$resptype) = @_;
    my $submission =$response->[&Apache::loncoursedata::RDs_submission()];
    $submission = &html_format_sub($submission,$resptype);
    my $correct = '';
    if ($ENV{'form.correctans'} eq 'true') {
        $correct = &Apache::lonstathelpers::get_student_answer
            ($prob,$student->{'username'},$student->{'domain'},
             $partid,$respid);
        $correct = &html_format_sub($correct,$resptype);
    }
    my $Str;
    $Str .= '<td valign="top">'.$submission.'</td>';
    if ($ENV{'form.correctans'} eq 'true') {
        $Str .= '<td valign="top">'.$correct.'</td>';
    }
    if ($ENV{'form.prob_status'} eq 'true') {
        $Str .= '<td valign="top">'.
            $response->[&Apache::loncoursedata::RDs_awarddetail()].
            '</td>';
        $Str .= '<td valign="top"><nobr>'.
            &Apache::lonlocal::locallocaltime
            ($response->[&Apache::loncoursedata::RDs_timestamp()]).
            '</nobr></td>';
        $Str .= '<td valign="top">'.
            $response->[&Apache::loncoursedata::RDs_tries()].
            '</td>';
        $Str .= '<td valign="top">'.
            $response->[&Apache::loncoursedata::RDs_awarded()].
            '</td>';
    }
    return $Str;
}

sub html_format_sub {
    my ($submission,$resptype) = @_;
    return '' if (! defined($submission) || $submission eq '');
    $submission = &HTML::Entities::decode($submission);
    $submission =~ s/\\\"/\"/g;
    $submission =~ s/\\\'/\'/g;
    if ($resptype eq 'essay') {
        $submission =~ s|\\r\\n|$/|g;
        $submission = &HTML::Entities::encode($submission,'<>&"');
        $submission =~ s|$/\s*$/|$/</p><p>$/|g;
        $submission =~ s|\\||g;
        $submission = '<p>'.$submission.'</p>';
    } elsif ($resptype eq 'radiobutton') {
        $submission = &HTML::Entities::encode($submission,'<>&"');
        $submission =~ s/=([^=])$//;
    } elsif ($resptype =~ /^(option|match|rank)$/) {
        $submission = 
            '<ul class="sub_studentans">'.
            '<li>'.join('</li><li>',
                        map { &HTML::Entities::encode($_,'<>&"'); 
                          } map { 
                              &Apache::lonnet::unescape($_) ;
                          } sort split('&',$submission)
                        ).
                        '</li><ul>';
    } else {
        $submission = &HTML::Entities::encode($submission,'<>&"');
    }
    return $submission;
}

#########################################################
#########################################################
##
##    Excel Output Routines
##
#########################################################
#########################################################
sub prepare_excel_output {
    my ($r,$Problems,$Students) = @_;
    my $c = $r->connection();
    #
    #
    # Determine the number of columns in the spreadsheet
    my $columncount = 3; # username, domain, id
    my $response_multiplier = 1;
    $response_multiplier ++   if ($ENV{'form.correctans'} eq 'true');
    $response_multiplier += 4 if ($ENV{'form.prob_status'} eq 'true');
    my $lastprob;
    foreach my $prob (@$Problems) {
        $columncount += ($response_multiplier * $prob->countResponses);
        last if ($columncount > 255);
        $lastprob = $prob;
    }
    if ($columncount > 255) {
        $r->print('<h1>'.&mt('Unable to complete request').'</h1>'.$/.
                  '<p>'.&mt('LON-CAPA is unable to produce your Excel spreadsheet because your selections will result in more than 255 columns.  Excel allows only 255 columns in a spreadsheet.').'</p>'.$/.
                  '<p>'.&mt('Consider selecting fewer problems to generate reports on, or reducing the number of items per problem.  Or use HTML or CSV output.').'</p>'.$/.
                  '<p>'.&mt('The last problem that will fit in the current spreadsheet is [_1].',$lastprob->compTitle).'</p>');
        $r->rflush();
        return;
    }
    #
    # Print out a message telling them what we are doing
    if (scalar(@$Problems) > 1) {
        $r->print('<h2>'.
                  &mt('Preparing Excel spreadsheet of student responses to [_1] problems',
                      scalar(@$Problems)).
                  '</h2>');
    } else {
        $r->print('<h2>'.
                  &mt('Preparing Excel spreadsheet of student responses').
                  '</h2>');
    }
    $r->rflush();
    #
    # Create the excel spreadsheet
    my $filename = '/prtspool/'.
        $ENV{'user.name'}.'_'.$ENV{'user.domain'}.'_'.
        time.'_'.rand(1000000000).'.xls';
    my $workbook  = Spreadsheet::WriteExcel->new('/home/httpd'.$filename);
    if (! defined($workbook)) {
        $r->log_error("Error creating excel spreadsheet $filename: $!");
        $r->print('<p>'.&mt("Unable to create new Excel file.  ".
                            "This error has been logged.  ".
                            "Please alert your LON-CAPA administrator").
                  '</p>');
        return undef;
    }
    #
    $workbook->set_tempdir('/home/httpd/perl/tmp');
    #
    my $format = &Apache::loncommon::define_excel_formats($workbook);
    my $worksheet  = $workbook->addworksheet('Student Submission Data');
    #
    # Add headers to the worksheet
    my $rows_output = 0;
    $worksheet->write($rows_output++,0,
                    $ENV{'course.'.$ENV{'request.course.id'}.'.description'},
                      $format->{'h1'});
    $rows_output++;
    my $cols_output = 0;
    my $title_row  = $rows_output++;
    my $partid_row = $rows_output++;
    my $respid_row = $rows_output++;
    my $header_row = $rows_output++;
    $worksheet->write($title_row ,0,'Problem Title',$format->{'bold'});
    $worksheet->write($partid_row,0,'Part ID',$format->{'bold'});
    $worksheet->write($respid_row,0,'Response ID',$format->{'bold'});
    # Student headers
    my @StudentColumns = ('username','domain','id');
    foreach (@StudentColumns) {
        $worksheet->write($header_row,$cols_output++,ucfirst($_),
                          $format->{'bold'});
    }
    # Problem headers
    foreach my $prob (@$Problems) {
        my $title = $prob->compTitle;
        $worksheet->write($title_row,$cols_output,
                          $title,$format->{'h3'});
        foreach my $partid (@{$prob->parts}) {
            $worksheet->write($partid_row,$cols_output,
                              $prob->part_display($partid));
            my $responses = [$prob->responseIds($partid)];
            my $resptypes = [$prob->responseType($partid)];
            for (my $i=0;$i<scalar(@$responses);$i++) {
                $worksheet->write($respid_row,$cols_output,
                                  $resptypes->[$i].', '.$responses->[$i]);
                $worksheet->write($header_row,$cols_output,'Submission');
                $cols_output++;
                if ($ENV{'form.correctans'} eq 'true') {
                    $worksheet->write($header_row,$cols_output,'Correct');
                    $cols_output++;
                }
                if ($ENV{'form.prob_status'} eq 'true') {
                    $worksheet->write($header_row,$cols_output++,
                                      'Award Detail');
                    $worksheet->write($header_row,$cols_output++,'Attempt');
                    $worksheet->write($header_row,$cols_output++,'Time');
                    $worksheet->write($header_row,$cols_output++,'Awarded');
                }
            }
        }
    }
    #
    # Populate the worksheet with the student data
    my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
        ($r,'Excel File Compilation Status',
         'Excel File Compilation Progress', 
         scalar(@$Students),'inline',undef,'Statistics','stats_status');
    my $max_row = $rows_output;
    foreach my $student (@$Students) {
        last if ($c->aborted());
        $cols_output = 0;
        my $student_row = $max_row;
        foreach my $field (@StudentColumns) {
            $worksheet->write($student_row,$cols_output++,
                              $student->{$field});
        }
        my $last_student_col = $cols_output-1;
        my $response_count;
        foreach my $prob (@$Problems) {
            foreach my $partid (@{$prob->parts}) {
                my @Response = $prob->responseIds($partid);
                my @ResponseType = $prob->responseType($partid);
                for (my $i=0;$i<=$#Response;$i++) {
                    my $response_start_col = $last_student_col + 
                        $response_count * $response_multiplier + 1;
                    $response_count++;
                    my $respid   = $Response[$i];
                    my $resptype = $ResponseType[$i];
                    my $results = 
                        &Apache::loncoursedata::get_response_data_by_student
                        ($student,$prob->symb(),$respid);
                    if (! defined($results)) {
                        $results = [];
                    }
                    #
                    $rows_output = $student_row;
                    #
                    for (my $j=scalar(@$results)-1;$j>=0;$j--) {
                        $cols_output = $response_start_col;
                        my $response = $results->[$j];
                        if ($ENV{'form.all_sub'} ne 'true') {
                            next if ($j ne scalar(@$results)-1);
                        }
                        my $cols_output = &write_excel_row
                            ($worksheet,
                             $rows_output++,$cols_output,
                             $response,$student,
                             $prob,$partid,$respid,
                             $format,$resptype);
                        if ($rows_output > $max_row) {
                            $max_row = $rows_output;
                        }
                    }
                }
            }
        }
        # Fill in the remaining rows with the students data
        for (my $row = $student_row+1;$row<=$max_row;$row++) {
            my $cols = 0;
            foreach my $field (@StudentColumns) {
                $worksheet->write($row,$cols++,
                                  $student->{$field});
            }
        }
        $rows_output++;
        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
                                                 'last student');
    }
    &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    #
    # Close the excel file
    $workbook->close();
    #
    # Write a link to allow them to download it
    $r->print('<p><a href="'.$filename.'">'.
              &mt('Your Excel spreadsheet.').
              '</a></p>'."\n");
    $r->print('<script>'.
              'window.document.Statistics.stats_status.value="'.
              'Done compiling spreadsheet.  See link below to download.'.
              '";</script>');
    $r->rflush();
    return;
}

sub write_excel_row {
    my ($worksheet,$row,$col,$response,$student,$prob,$partid,$respid,
        $format,$resptype) = @_;
    # 
    my $submission =$response->[&Apache::loncoursedata::RDs_submission()];
    $submission = &excel_format_response($submission,$resptype);
    $worksheet->write($row,$col++,$submission);
    if ($ENV{'form.correctans'} eq 'true') {
        my $correct = &Apache::lonstathelpers::get_student_answer
            ($prob,$student->{'username'},$student->{'domain'},
             $partid,$respid);
        $correct =&excel_format_response($correct,$resptype);
        $worksheet->write($row,$col++,$correct);
    }
    if ($ENV{'form.prob_status'} eq 'true') {
        $worksheet->write
            ($row,$col++,
             $response->[&Apache::loncoursedata::RDs_awarddetail()]);
        $worksheet->write
            ($row,$col++,$response->[&Apache::loncoursedata::RDs_tries()]);
        $worksheet->write
            ($row,$col++,
             &Apache::lonstathelpers::calc_serial
             ($response->[&Apache::loncoursedata::RDs_timestamp()]),
             $format->{'date'});
        $worksheet->write
            ($row,$col++,$response->[&Apache::loncoursedata::RDs_awarded()]);
    }
    return $col;
}

sub excel_format_response {
    my ($answer,$responsetype) = @_;
    if ($responsetype eq 'radiobutton') {
        $answer =~ s/=([^=])$//;
    } elsif ($responsetype =~ /^(option|match)$/) {
        $answer = join("\n",
                       map { 
                           &Apache::lonnet::unescape($_) ;
                       } sort split('&',$answer)
                       );
    } elsif ($responsetype eq 'string') {
        $answer =~ s/\\(n|r)/\n/g;
        $answer =~ s/(\s*$|^\s*)//g;
        $answer =~ s/\\\'/\'/g;
    }
    if ($answer =~ m/^=/) {
        $answer = ' '.$answer;
    }
    return $answer;
}

#########################################################
#########################################################
##
##      CSV output of student answers
##
#########################################################
#########################################################
sub prepare_csv_output {
    my ($r,$problems,$students) = @_;
    my $c = $r->connection();
    #
    $r->print('<h2>'.
              &mt('Generating CSV report of student responses').'</h2>');
    #
    # Progress window
    my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
        ($r,'CSV File Compilation Status',
         'CSV File Compilation Progress', 
         scalar(@$students),'inline',undef,'Statistics','stats_status');
    
    $r->rflush();
    #
    # Open a file
    my $outputfile;
    my $filename = '/prtspool/'.
        $ENV{'user.name'}.'_'.$ENV{'user.domain'}.'_'.
            time.'_'.rand(1000000000).'.csv';
    unless ($outputfile = Apache::File->new('>/home/httpd'.$filename)) {
        $r->log_error("Couldn't open $filename for output $!");
        $r->print("Problems occured in writing the csv file.  ".
                  "This error has been logged.  ".
                  "Please alert your LON-CAPA administrator.");
        $outputfile = undef;
    }
    #
    # Compute the number of columns per response
    my @response_headers = ('Submission');
    if ($ENV{'form.correctans'} eq 'true') {
        push(@response_headers,'Correct');
    } 
    if ($ENV{'form.prob_status'} eq 'true') {
        push(@response_headers,'Award Detail'); 
        push(@response_headers,'Time');
        push(@response_headers,'Attempt');
        push(@response_headers,'Awarded');
    }
    my $response_multiplier = scalar(@response_headers);
    #
    # Create the table header
    my @student_columns = ('username','domain','id');
    #
    my %headers;
    push(@{$headers{'student'}},@student_columns);
    # Pad for the student data
    foreach my $row ('problem','part','response') {
        foreach (@student_columns) {
            push(@{$headers{$row}},'');
        }
    }
    #
    # we put the headers into the %headers hash
    my $prob_start_idx = 0;
    foreach my $prob (@$problems) {
        $headers{'problem'}->[$prob_start_idx] = $prob->compTitle;
        my $part_start_idx = $prob_start_idx;
        foreach my $partid (@{$prob->parts}) {
            $headers{'part'}->[$part_start_idx] = $prob->part_display($partid);
            my $responses = [$prob->responseIds($partid)];
            for (my $i=0;$i<scalar(@$responses);$i++) {
                my $resp_idx = $prob_start_idx + $response_multiplier * $i;
                $headers{'response'}->[$resp_idx]=
                    &mt('Response [_1]',$responses->[$i]);
                for (my $j=0;$j<=$#response_headers;$j++) {
                    $headers{'student'}->[$resp_idx+$j]=$response_headers[$j];
                }
            }
            $part_start_idx += scalar(@$responses)*$response_multiplier;
        }
        $prob_start_idx += $prob->countResponses * $response_multiplier;
    }
    foreach my $row ('problem','part','response','student') {
        print $outputfile '"'.join('","',map { ''; } @student_columns).'","'.
            join('","',
                 map { 
                     &Apache::loncommon::csv_translate($_); 
                 } @{$headers{$row}}).'"'.$/;
    }
    #
    # Main loop
    foreach my $student (@$students) {
        last if ($c->aborted());
        my @rows;
        my $prob_start_idx = 0;
        foreach my $prob (@$problems) {
            my $part_start_idx = 0;
            foreach my $partid (@{$prob->parts}) {
                my @responses = $prob->responseIds($partid);
                my @response_type = $prob->responseType($partid);
                for (my $i=0;$i<=$#responses;$i++) {
                    my $resp_start_idx = $response_multiplier * $i;
                    my $respid   = $responses[$i];
                    my $results = 
                        &Apache::loncoursedata::get_response_data_by_student
                        ($student,$prob->symb(),$respid);
                    if (! defined($results)) {
                        $results = [];
                    }
                    for (my $j=0; $j<scalar(@$results);$j++) {
                        if ($ENV{'form.all_sub'} ne 'true') {
                            next if ($j != 0);
                        }
                        my $idx = scalar(@$results) - $j - 1;
                        my $response = $results->[$idx];
                        my @data = &compile_response_data($response,$student,
                                                          $prob,$partid,
                                                          $respid);
                        for (my $k=0;$k<=$#data;$k++) {
                            $rows[$j]->[$prob_start_idx + $part_start_idx +
                                        $resp_start_idx + $k] = $data[$k];
                        }
                    }
                }
                $part_start_idx += scalar(@responses)*$response_multiplier;
            }
            $prob_start_idx += $prob->countResponses * $response_multiplier;
        }
        foreach my $row (@rows) {
            print $outputfile '"'.join('","',
                                       map { $student->{$_}; }
                                       @student_columns).'"';
                                           
            for (my $i=0;$i<$prob_start_idx;$i++) {
                my $value = &Apache::loncommon::csv_translate($row->[$i]);
                $value ||='';
                print $outputfile ',"'.$value.'"';
            }
            print $outputfile $/;
        }
        undef(@rows);
        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
                                                 'last student');
    }
    close($outputfile);
    #
    # Close the progress window
    &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    #
    # Tell the user where to get their csv file
    $r->print('<br />'.
              '<a href="'.$filename.'">'.&mt('Your csv file.').'</a>'."\n");
    $r->rflush();
    return;
}

sub compile_response_data {
    my ($response,$student,$prob,$partid,$respid) = @_;
    my @rowdata;
    push(@rowdata,$response->[&Apache::loncoursedata::RDs_submission()]);
    if ($ENV{'form.correctans'} eq 'true') {
        my $correct = &Apache::lonstathelpers::get_student_answer
            ($prob,$student->{'username'},$student->{'domain'},
             $partid,$respid);
        push(@rowdata,$correct);
    }
    if ($ENV{'form.prob_status'}) {
        push(@rowdata,$response->[&Apache::loncoursedata::RDs_awarddetail()]);
        push(@rowdata,&Apache::lonlocal::locallocaltime
             ($response->[&Apache::loncoursedata::RDs_timestamp()]));
        push(@rowdata,$response->[&Apache::loncoursedata::RDs_tries()]);
        push(@rowdata,$response->[&Apache::loncoursedata::RDs_awarded()]);
    }
    return @rowdata;
}

#########################################################
#########################################################
##
##   Generic Interface Routines
##
#########################################################
#########################################################
sub CreateInterface {
    ##
    ## Output Selection
    my $output_selector = $/.'<select name="output">'.$/;
    foreach ('HTML','Excel','CSV') {
        $output_selector .= '    <option value="'.lc($_).'"';
        if ($ENV{'form.output'} eq lc($_)) {
            $output_selector .= ' selected ';
        }
        $output_selector .='>'.&mt($_).'</option>'.$/;
    } 
    $output_selector .= '</select>'.$/;
    ##
    ## Environment variable initialization
    my $Str = '';
    $Str .= &Apache::lonhtmlcommon::breadcrumbs
        (undef,'Student Submission Reports');
    $Str .= '<p>';
    $Str .= '<table cellspacing="5">'."\n";
    $Str .= '<tr>';
    $Str .= '<th>'.&mt('Sections').'</th>';
    $Str .= '<th>'.&mt('Enrollment Status').'</th>';
    $Str .= '<th>'.&mt('Output as [_1]',$output_selector).'</th>';
    $Str .= '</tr>'."\n";
    #
    $Str .= '<tr><td align="center">'."\n";
    $Str .= &Apache::lonstatistics::SectionSelect('Section','multiple',5);
    $Str .= '</td>';
    #
    $Str .= '<td align="center">';
    $Str .= &Apache::lonhtmlcommon::StatusOptions(undef,undef,5);
    $Str .= '</td>';
    #
    # Render problem checkbox
    my $prob_checkbox = '<input type="checkbox" name="renderprob" ';
    if (exists($ENV{'form.renderprob'}) && $ENV{'form.renderprob'} eq 'true') {
        $prob_checkbox .= 'checked ';
    }
    $prob_checkbox .= 'value="true" />';
    #
    # Compute correct answers checkbox
    my $ans_checkbox = '<input type="checkbox" name="correctans" ';
    if (exists($ENV{'form.correctans'}) && $ENV{'form.correctans'} eq 'true') {
        $ans_checkbox .= 'checked ';
    }
    $ans_checkbox .= 'value="true" />';
    #
    # Show all submissions checkbox
    my $all_sub_checkbox = '<input type="checkbox" name="all_sub" ';
    if (exists($ENV{'form.all_sub'}) && 
        $ENV{'form.all_sub'} eq 'true') {
        $all_sub_checkbox .= 'checked ';
    }
    $all_sub_checkbox.= 'value="true" />';
    #
    # problem status checkbox
    my $prob_status_checkbox = '<input type="checkbox" name="prob_status" ';
    if (exists($ENV{'form.prob_status'}) && 
        $ENV{'form.prob_status'} eq 'true') {
        $prob_status_checkbox .= 'checked ';
    }
    $prob_status_checkbox .= 'value="true" />';
    #
    $Str .= '<td align="right" halign="top">'.
        '<label><b>'.
        &mt('Show problem [_1]',$prob_checkbox).'</b></label><br />'.
        '<label><b>'.
        &mt('Show correct answers [_1]',$ans_checkbox).'</b></label><br />'.
        '<label><b>'.
        &mt('Show all submissions [_1]',$all_sub_checkbox).
        '</b></label><br />'.
        '<label><b>'.
        &mt('Show problem grading [_1]',$prob_status_checkbox).
        '</b></label><br />'.
        '</td>';
    #
    $Str .= '</tr>'."\n";
    $Str .= '</table>'."\n";
    #
    $Str .= '<nobr>'.&mt('Status: [_1]',
                         '<input type="text" '.
                         'name="stats_status" size="60" value="" />').
            '</nobr>'.'</p>';    
    ##
    return $Str;
}

1;

__END__
